#!/usr/bin/env python

import os
import logging
import logging.handlers
from os.path import *
from commands import *

from OVSSiteCluster import cluster_prepare_ocfs2_conf
from OVSXCluster import *
from OVSServerProxy import ServerProxy
from OVSProtocol import *
from OVSCommons import *
from OVSCDS import *
from OVSXSysInfo import get_agent_version
import utils.do_rpc
from _agtctlutil import *

ADMIN_SP = ServerProxy(utils.do_rpc.get_local_agt_url())

#======================================================================
#setup the logging system, no dependency to the log server.

formatter = logging.Formatter(fmt="%(asctime)s %(levelname)s=> %(message)s", datefmt="%Y-%m-%d %H:%M:%S")
handler = logging.handlers.RotatingFileHandler(join(LOG_DIR, "ovs_autorun.log"), "a", 1024*1024, 3)
handler.setFormatter(formatter)
logging.getLogger("").setLevel(logging.DEBUG)
logging.getLogger("").addHandler(handler)

#======================================================================
#user defined startup function

def prepare_cluster_root():
    """Check if root_sro exists in LOCAL_DB,node.
    if exists:
        root_sr.initialize()
    else: #operate in single node mode.
        ln -sf CONFIG_DIR CONFIG_DIR_CLUSTER
    """
    try:
        #remove invalid link if necessary
        if not exists(OVS_BASE) and islink(OVS_BASE):
            os.remove(OVS_BASE)

        rs = ADMIN_SP.clusterm_init_root_sr()
        rs = DJSON(rs)
        if rs["status"] == OP_STATUS_FAIL:
            raise OVSException(error=rs["error"])

        db_save("LOCAL_DB,node", "local_db", False)

        d = db_dump("srv")
        if d:
            init_root = db_load("cluster", "init_root", get=True)
            if init_root:
                ha_enable = db_load("cluster", "pool_ha_enable", get=True)
                ha_enable = ha_enable or False
                ADMIN_SP.cluster_setup(ha_enable)
            else:
                ADMIN_SP.cluster_init_sr()

        logging.info("prepare_cluster_root: cluster conf.")
    except OVSException, e:
        if not e.errcode == ERR_INTERNAL_ERROR:
            raise e

        if exists(OVS_BASE):
            if islink(OVS_BASE):
                os.remove(OVS_BASE)
            else:
                if islink(CONFIG_DIR_CLUSTER) and os.readlink(CONFIG_DIR_CLUSTER) == CONFIG_DIR:
                    db_save("LOCAL_DB,node", "local_db", True)
                    logging.info("prepare_cluster_root: local conf.")
                else:
                    ovs_backup(OVS_BASE)

        if not exists(OVS_BASE):
            os.makedirs(OVS_BASE)
            cmd = ['ln', '-sf', CONFIG_DIR, CONFIG_DIR_CLUSTER]
            try:
                run_cmd(cmd)
                db_save("LOCAL_DB,node", "local_db", True)
                logging.info("prepare_cluster_root: local conf.")
            except Exception, e:
                errmsg = fmt_err_msg(e)
                logging.error("perpare_cluster_root: error. => %s" % errmsg)


def prepare_cluster_heartbeat():
    try:
        d = db_dump("srv")
        if d:
            # prepare cluster conf
            cluster_prepare_ocfs2_conf()
            # start o2cb
            clusterm_start_o2cb_service()
            # start heartbeat
            pool_ha_enable = db_load("cluster", "pool_ha_enable", get=True)
            if pool_ha_enable:
                rs = clusterm_start_heartbeat()
                GETVAL(rs)
        else:
            logging.debug("prepare_cluster_heartbeat: no server.")
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logging.error("prepare_cluster_heartbeat: => %s" % errmsg)


#======================================================================
#autorun jobs
JOBS = [
    ["command", join(dirname(__file__), "utils/upgrade_agent.py")],
    ["function", "get_agent_version", ()],
    ["function", "prepare_cluster_root", ()],
    ["function", "prepare_cluster_heartbeat", ()],
    ["command", join(dirname(__file__), "utils/upgrade.py")],
    ["command_bg", join(dirname(__file__), "OVSRemasterServer.py")],
    ["command_bg", join(dirname(__file__), "OVSMonitorServer.py")],
    ["command_bg", join(dirname(__file__), "OVSPolicyServer.py")],
    ["do_rpc", "echo", ("what",)],
    ["function", "do_shutdown_mode", ("leave",)],
]

def main():
    logging.info("----------------------AUTO START-------------------------")
    for job in JOBS:
        try:
            logging.info("%s" % (repr(job),))
            if job[0] == "do_rpc":
                operation, param = job[1:]
                getattr(ADMIN_SP, operation)(*param)
            elif job[0] == "command":
                cmd = job[1]
                old_run_cmd(cmd)
            elif job[0] == "command_bg":
                cmd = job[1]
                run_cmd_bg(cmd)
            elif job[0] == "function":
                func_name, param = job[1:]
                eval(func_name)(*param)
        except Exception, e:
            errmsg = fmt_err_msg(e)
            logging.error("%s => %s" % (repr(job), errmsg))
    logging.info("--------------------------END----------------------------")

if __name__ == "__main__":
    main()
