#!/usr/bin/env python
#^#
# OVSDB.py
#
# <description>
#
#
# Copyright (C) 2006, 2007 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.
#$#

from os.path import basename, exists, abspath
from OVSXUtility import find_real_path
from OVSConstants import DEFAULT_AGENT_USERNAME, DEFAULT_AGENT_PASSWD, DEFAULT_AGENT_PORT
from OVSCommons import get_host_by_name
from OVSCDS import CDS

_g_vm_default = {
    "vtech": "xen",
    "status": "DOWN",
    "ip":"",
    "agt_addr":"",
    "agt_username": DEFAULT_AGENT_USERNAME,
    "agt_passwd": DEFAULT_AGENT_PASSWD,
    "agt_port": DEFAULT_AGENT_PORT,
    "readlock": 0,
    "host_srv": "",
    "real_path": "",
    }

#----------------------------<> vm <>----------
def db_get_vm(vm_path):
    cds = CDS("vm", "vmp")
    vm_path = abspath(vm_path)
    try:
        if vm_path in cds.db_vm:
            vm = cds.db_vm[vm_path]
        else:
            cds.db_vm[vm_path] = _g_vm_default.copy()
            vm = cds.db_vm[vm_path]
            #name->path index
            vm_name = basename(vm_path)
            cds.db_vmp[vm_name] = vm_path
    finally:
        cds.close()
    return vm

def db_update_vm(vm_path, d):
    cds = CDS("vm", "vmp")
    vm_path = abspath(vm_path)
    try:
        if vm_path in cds.db_vm:
            vm = cds.db_vm[vm_path]
            vm.update(d)
        else:
            vm = _g_vm_default.copy()
            vm.update(d)
        cds.db_vm[vm_path] = vm
        #name->path index
        vm_name = basename(vm_path)
        cds.db_vmp[vm_name] = vm_path
    finally:
        cds.close()
    return vm

def db_del_vm(vm_path):
    cds = CDS("vm", "vmp")
    vm_path = abspath(vm_path)
    try:
        if vm_path in cds.db_vm:
            del cds.db_vm[vm_path]
        #name->path index
        vm_name = basename(vm_path)
        if vm_name in cds.db_vmp:
            del cds.db_vmp[vm_name]
    finally:
        cds.close()

def db_chkset_vm_status(vm_path, status_before, status_after):
    cds = CDS("vm", "vmp")
    vm_path = abspath(vm_path)
    try:
        if vm_path in cds.db_vm:
            vm = cds.db_vm[vm_path]
        else:
            cds.db_vm[vm_path] = _g_vm_default.copy()
            vm = cds.db_vm[vm_path]
            #name->path index
            vm_name = basename(vm_path)
            cds.db_vmp[vm_name] = vm_path

        if status_before:
            if vm["status"] not in status_before:
                errmsg = "vm('%s') status ('%s') not in %s" % \
                        (vm_path, vm['status'], repr(status_before))
                raise Exception(errmsg)
            #accquire read lock
            elif status_after == "READLOCK":
                #vm status may out of sync with lock, resync
                if vm["status"] == "DOWN":
                    vm["readlock"] = 0
                vm["readlock"] += 1
                vm['status'] = status_after
            #release read lock
            #include DOWN in status_before, just be more torlerance
            elif vm["status"] in ("READLOCK", "DOWN") and status_after == "DOWN":
                if vm["readlock"] > 0: vm["readlock"] -= 1
                if vm["readlock"] == 0: vm['status'] = "DOWN"
            else:
                vm['status'] = status_after
        else:
            #if not status_before
            vm['status'] = status_after

        cds.db_vm[vm_path] = vm
    finally:
        cds.close()

#----------------------------<> server <>----------
def db_get_srv(hostname):
    cds = CDS("srv", "srvp")
    try:
        if hostname in cds.db_srv:
            srv = cds.db_srv[hostname]
        else:
            srv = None
            ip = get_host_by_name(hostname)
            if ip in cds.db_srvp:
                _hostname = cds.db_srvp[ip]
                if _hostname in cds.db_srv:
                    srv = cds.db_srv[_hostname]
            if not srv:
                for _hostname in cds.db_srv:
                    if get_host_by_name(_hostname) == ip:
                        srv = cds.db_srv[_hostname]
                        cds.db_srvp[ip] = _hostname
                        break
            if not srv:
                raise Exception("srv('%s') not found in db!" % hostname)
    finally:
        cds.close()

    return srv

def db_update_srv(hostname, d, insert=False):
    cds = CDS("srv", "srvp")
    try:
        if hostname in cds.db_srv:
            srv = cds.db_srv[hostname]
            srv.update(d)
            cds.db_srv[hostname] = srv
        else:
            srv = None
            ip = get_host_by_name(hostname)
            if ip in cds.db_srvp:
                _hostname = cds.db_srvp[ip]
                if _hostname in cds.db_srv:
                    srv = cds.db_srv[_hostname]
                    srv.update(d)
                    cds.db_srv[_hostname] = srv
            if not srv:
                for _hostname in cds.db_srv:
                    if get_host_by_name(_hostname) == ip:
                        srv = cds.db_srv[_hostname]
                        srv.update(d)
                        cds.db_srv[_hostname] = srv
                        cds.db_srvp[ip] = _hostname
                        break
            if not srv:
                if insert:
                    cds.db_srv[hostname] = d
                    cds.db_srvp[ip] = hostname
                else:
                    raise Exception("srv('%s') not found in db!" % hostname)
    finally:
        cds.close()

def db_del_srv(hostname):
    cds = CDS("srv", "srvp")
    try:
        #del all server with same ip
        ip = get_host_by_name(hostname)

        tgt_hostnames = []
        for _hostname in cds.db_srv:
            if get_host_by_name(_hostname) == ip:
                tgt_hostnames.append(_hostname)
        for _hostname in tgt_hostnames:
            del cds.db_srv[_hostname]

        if ip in cds.db_srvp:
            del cds.db_srvp[ip]
    finally:
        cds.close()

#------------------------------<> vm path mapping <>------
def db_get_real_vm_path(vm_path):
    """
    NOTE: deprecated!!
    """
    real_path = ""
    vm = db_get_vm(vm_path)
    if "real_path" in vm and vm["real_path"] and exists(vm["real_path"]):
        real_path = vm["real_path"]
    else:
        #if not exist, find_real_path will raise Exception
        real_path = find_real_path(vm_path)
        db_update_vm(vm_path, {"real_path": real_path})
    return real_path

def db_get_vm_path(vm_name):
    vm_path = ""
    cds = CDS("vm", "vmp")
    try:
        if vm_name in cds.db_vmp:
            vm_path = cds.db_vmp[vm_name]
        else: #init db_vmp with db_vm
            for vp in cds.db_vm:
                name = basename(vp)
                if name not in cds.db_vmp:
                    cds.db_vmp[name] = vp
            if vm_name in cds.db_vmp:
                vm_path = cds.db_vmp[vm_name]
    finally:
        cds.close()
    return vm_path

#------------------------------<> vm path mapping <>------

