#!/usr/bin/env python

#^#
#
#
# Copyright (C) 2006, 2007 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.
#$#

import traceback
import sys
from string import Template
from StringIO import StringIO

from OVSExceptionLookup import *
from OVSConfig import get_config

class OVSException(Exception):
    def __init__(self, errcode=ERR_RAW_MESSAGE, argd=None, error=None):
        #when reconstruct an exception from return error value.
        if error:
            self.errcode, self.errmsg = error["errcode"], error["errmsg"]
            return

        self.errcode = str(errcode)
        err = ERROR_DICT.get(self.errcode)
        if err:
            if not argd: argd = {}
            self.errmsg = Template(err[0]).safe_substitute(argd)
        else:
            self.errmsg = ""

    def __str__(self):
        return "errcode=%s, errmsg=%s" % (self.errcode, self.errmsg)

    def iskindof(self, errgroup):
        l = ERRGRP_DICT.get(errgroup)
        if l and self.errcode in l:
            return True
        return False

    @property
    def value(self):
        return {
            "errcode"   : self.errcode,
            "errmsg"    : self.errmsg,
        }

#----------------------------------------------------------------------
def fmt_err_msg(e):
    if isinstance(e, OVSException):
        errmsg = str(e)
    else: #none OVSException
        errmsg = "<%s: %s>" % (e.__class__.__name__, e)

    #append stacktrace to error message
    try:
        config = get_config()
        if config.get("debug", "traceback").startswith("enable"):
            tb = sys.exc_info()[2]
            if tb:
                sio = StringIO()
                try:
                    traceback.print_tb(tb, file=sio)
                    errmsg = "%s\n\nStackTrace:\n%s" % (errmsg, sio.getvalue())
                finally:
                    sio.close()
    except Exception, e:
        _errmsg = "<%s: %s>" % (e.__class__.__name__, e)
        print _errmsg

    return errmsg
