#!/usr/bin/env python

import re
import os
import crypt
from tempfile import mkdtemp
from os.path import exists, join, abspath, basename, dirname
from commands import getstatusoutput


cleanUpCommands = "echo"
loop_device="/dev/loop0"


def _debug(msg):
    print "DEBUG: %s" % repr(msg)


def _run_cmd(cmd):
    _debug(cmd)
    if not cmd.strip():
        return -1, "No command found."
    (status, output) = getstatusoutput(cmd)
    if status != 0:
        print "cmd: '%s' => %s" % (cmd, output)
    return status, output


def assignAvailableLoopBackDevice():
    cmd = "losetup -f"
    status, output = _run_cmd(cmd)
    return output


def mountLVMPartition(mountSource, mountDestination, device):
    lvm_dir = join(dirname(mountSource), "lvm")
    lvm_conf_file = join(lvm_dir, "lvm.conf")
    _run_cmd("mkdir -p %s" % lvm_dir)

    global loop_device
    config = """devices {
    dir = "%s"
    scan = [ "/dev/mapper" ]
    filter = [ 'a|/dev/mapper/%sp*|', 'r|.*|' ]
}
""" % (lvm_dir, basename(loop_device))
    f = open(lvm_conf_file, "w")
    try:
        f.write(config)
    finally:
        f.close()

    os.environ["LVM_SYSTEM_DIR"] = lvm_dir
    lines = _run_cmd("vgscan")[1].splitlines()
    last_line = lines[-1]
    vg = last_line.split("\"")[1]
    _run_cmd("vgchange -ay %s" % vg)

    status, output = _run_cmd("lvs | grep %s" % vg)
    lines = output.splitlines()

    for line in lines:
        lv = line.strip().split()[0]
        cmd = "mount %s/%s/%s %s" % (lvm_dir, vg, lv, mountDestination)
        _run_cmd(cmd)
        if exists("%s/etc/redhat-release" % mountDestination):
            global cleanUpCommands
            umount_cmd = "umount %s;" % mountDestination
            cleanUpCommands = "%s %s vgchange -an %s; rm -rf %s;" % (umount_cmd, umount_cmd, vg, lvm_dir)
            return True
        else:
            _run_cmd("umount %s" % mountDestination)

    _run_cmd("vgchange -an %s; rm -rf %s" % (vg, lvm_dir))
    return False


def mountPartition(mountSource, mountDestination, device):
    status, output = _run_cmd("file -s %s" % device)
    if "LVM" in output:
        return mountLVMPartition(mountSource, mountDestination, device)
    else:
        status, output = _run_cmd("mount %s %s" % (device, mountDestination))
        if status == 0 and exists("%s/etc/redhat-release" % mountDestination):
            global cleanUpCommands
            umount_cmd = "umount %s;" % mountDestination
            cleanUpCommands = "%s %s" % (umount_cmd, umount_cmd)
            return True
        else:
            return False


def mountDisk(mountSource, mountDestination, loop_device):
    cmd = "kpartx -av %s" % loop_device
    status, output = _run_cmd(cmd)
    for line in output.splitlines():
        if not line.strip():
            continue
        device = "/dev/mapper/" + line.split()[2]
        if mountPartition(mountSource, mountDestination, device):
            global cleanUpCommands
            cleanUpCommands = "%s kpartx -d %s" % (cleanUpCommands, loop_device)
            return True

    _run_cmd("kpartx -d %s" % loop_device)
    return False


OVS_AUTOSTART = "/etc/ovs-autostart"


def copyConfigIntoImage(src, image):
    global cleanUpCommands
    global loop_device
    try:
        src = abspath(src)
        image = abspath(image)
        loop_device = assignAvailableLoopBackDevice()
        mountDestination = mkdtemp()
        _run_cmd("mkdir -p %s" % mountDestination)
        _run_cmd("losetup %s %s" % (loop_device, image))

        ret = False
        if mountDisk(image, mountDestination, loop_device):
            _run_cmd("rsync -a --no-p %s/ %s" % (src, mountDestination))

            etc_rc_local = "%s/etc/rc.local" % mountDestination
            autostart_cmd = "for i in $(ls %s); do /bin/bash %s/$i; done" % (OVS_AUTOSTART, OVS_AUTOSTART)
            grep_cmd = "grep '%s' %s" % (autostart_cmd, etc_rc_local)
            status, output = _run_cmd(grep_cmd)
            if status != 0:
                _run_cmd("echo '%s' >> %s" % (autostart_cmd, etc_rc_local))
            ret = True

        cleanUpCommands = "%s; losetup -d %s" % (cleanUpCommands, loop_device)
        _run_cmd(cleanUpCommands)
        if not os.listdir(mountDestination):
            _run_cmd("rm -rf %s" % mountDestination)
        return ret

    except Exception, e:
            print e
            cleanUpCommands = "%s; losetup -d %s" % (cleanUpCommands, loop_device)
            _run_cmd(cleanUpCommands)
            if not os.listdir(mountDestination):
                _run_cmd("rm -rf %s" % mountDestination)
            return False


def parse_config(value):
    def split_child(value):
        if not "{" in value: return []

        value = "".join(value.split())
        l = []
        begin, idx = 0, 0

        while begin < len(value):
            while value[idx] != "=":
                idx += 1
            c_name = value[begin:idx]
            c_name = c_name.strip(",")

            while value[idx] != "{":
                idx += 1
            begin, curly_count = idx, 1
            while curly_count != 0:
                idx += 1
                if value[idx] == "{": curly_count += 1
                elif value[idx] == "}": curly_count -= 1
            c_value = value[begin+1:idx]

            l.append((c_name, c_value))
            begin = idx + 1

        return l

    l = split_child(value)
    if not l:
        return dict([x.split("=") for x in value.split(",") if x])

    r = {}
    for c_key, c_value in l:
        r[c_key] = parse_config(c_value)

    return r


def createPasswordConfiguration(password_dict, config_path):
    user = password_dict["USER"]
    password = password_dict["PASSWORD"]
    crypted_password = crypt.crypt(password, "Xa")
    cmd = """#!/bin/bash
usermod -p %s %s
rm $0
""" % (crypted_password, user)

    _dir = join(config_path, OVS_AUTOSTART.strip("/"))
    _run_cmd("mkdir -p %s" % _dir)
    passwd_file = join(_dir, "passwd_file")

    f = open(passwd_file, "w")
    try:
        f.write(cmd)
    finally:
        f.close()


NETWORK_SCRIPTS_DIR = "/etc/sysconfig/network-scripts"
NETWORK_SCRIPT_PREFIX = "ifcfg-eth"
RESOLV_CONF = "/etc/resolv.conf"
ETC_HOSTS = "/etc/hosts"
NETWORK_IDENTITY = "/etc/sysconfig/network"
CONFIG_STATIC  = "STATIC"
CONFIG_DYNAMIC = "DYNAMIC"


def createNetworkInterfaceConfig(NIC_dict, id, file_name):
    if NIC_dict["CONFIGURATION_TYPE"] == CONFIG_DYNAMIC:
        config = """DEVICE=eth%s
ONBOOT=yes
BOOTPROTO=dhcp
""" % id
    else:
        config = """DEVICE=eth%s
IPADDR=%s
NETMASK=%s
ONBOOT=yes
BOOTPROTO=static
""" % (id, NIC_dict["IP_ADDRESS"], NIC_dict["SUBNET_MASK"])

    f = open(file_name, "w")
    try:
        f.write(config)
    finally:
        f.close()


def createResolvConfig(NIC_dict, file_name):
    dns_servers = NIC_dict["DNS_SERVER"].split("/")

    f = open(file_name, "a")
    try:
        for server in dns_servers:
            f.write("nameserver %s\n" % server)
    finally:
        f.close()


def createEtcHostsConfig(NIC_dict, file_name):
    config = """127.0.0.1\tlocalhost.localdomain\tlocalhost
%s\t%s.%s\t%s
""" % (NIC_dict["IP_ADDRESS"], NIC_dict["HOST_NAME"], NIC_dict["DOMAIN_NAME"], NIC_dict["HOST_NAME"])

    f = open(file_name, "a")
    try:
        f.write(config)
    finally:
        f.close()


def createNetworkIdentity(NIC_dict, file_name):
    config = """HOSTNAME=%s.%s
GATEWAY=%s
""" % (NIC_dict["HOST_NAME"], NIC_dict["DOMAIN_NAME"], NIC_dict["DEFAULT_GATEWAY"])

    f = open(file_name, "a")
    try:
        f.write(config)
    finally:
        f.close()


def createNetworkConfiguration(network_config, config_path):
    resolv_conf = join(config_path, RESOLV_CONF.strip("/"))
    etc_hosts = join(config_path, ETC_HOSTS.strip("/"))
    network_identity = join(config_path, NETWORK_IDENTITY.strip("/"))

    _dir = join(config_path, NETWORK_SCRIPTS_DIR.strip("/"))
    _run_cmd("mkdir -p %s" % _dir)
    for file_name in resolv_conf, etc_hosts, network_identity:
        _run_cmd("rm -f %s; touch %s" % (file_name, file_name))

    for k, v in network_config.items():
        id = int(k[3:])
        _script = "%s%s" % (NETWORK_SCRIPT_PREFIX, id)
        network_script = join(_dir, _script)
        createNetworkInterfaceConfig(v, id, network_script)

        if v["CONFIGURATION_TYPE"] == CONFIG_STATIC:
            if id == 0:
                createNetworkIdentity(v, network_identity)
            createResolvConfig(v, resolv_conf)
            createEtcHostsConfig(v, etc_hosts)


def validateConfig(config):
    if "USER_PASSWORD" in config:
        user_password = config["USER_PASSWORD"]
        if not user_password.keys().sort() == ["USER", "PASSWORD"].sort():
            return False

    if "NIC" in config:
        nic = config["NIC"]
        for k, v in nic.items():
            m = re.match(r"^eth\d+$", k)
            if not m:
                return False
            if not "CONFIGURATION_TYPE" in v:
                return False
            type = v["CONFIGURATION_TYPE"]
            if not (type == CONFIG_STATIC or type == CONFIG_DYNAMIC):
                return False
    return True


def createConfigurations(value, config_path):
    d = parse_config(value)
    if not validateConfig(d): return False

    if "USER_PASSWORD" in d:
        createPasswordConfiguration(d["USER_PASSWORD"], config_path)
    if "NIC" in d:
        createNetworkConfiguration(d["NIC"], config_path)

    return True


def cleanConfigurations(config_path):
    status, output = _run_cmd("rm -rf %s" % config_path)
    if status != 0: return False
    return True


def createPropFile(props, prop_file):
    status, output = _run_cmd("mkdir -p %s" % dirname(prop_file))
    if status != 0: return False
    l = props.split(";")
    f = open(prop_file, "w")
    try:
        for e in l:
            if e.strip(): f.write("%s\n" % e)
    finally:
        f.close()
    return True

