"""
OVSOVF.py Oracle VM Open-OVF
Copyright (C) 2006-2009  Oracle.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""

from OVSWrappers import *
from ovf.OvfSet import OvfSet
from ovf.OvfSet import FORMAT_TAR

import os
import tarfile

class _OVSOVFManagement(object):
    seed_pool_path = '/OVS/seed_pool'
    
    def __init__(self, pool_path=None, **kwargs):
        if pool_path:
            self.seed_pool_path = pool_path

    
    def get_available_ovas(self):
        '''
        Get a list of ova files
        
        @return: a list of ova filename
        '''
        filenames = os.listdir(self.seed_pool_path)
        return [ filename for filename in filenames 
                if filename.endswith('.ova')
                and os.path.isfile(os.path.join(self.seed_pool_path, filename)) ]

class _OVSOvfSet(OvfSet):
    tmpdir = None
    path = None

    def __init__(self, path, **kwargs):
        '''
        @type path: String
        @param path: path of ova file
        '''
        super(_OVSOvfSet, self).__init__(None, 'r')

        self.path = path
        self.tf = tarfile.open(self.path, 'r')
                
    def validate(self):
        '''
        Validate ova package.

        @rtype: Boolean
        @return: True if all the checksums of the files match
                 False if at least one checksum does not match

        @raise IOError: File does not exist at path
        '''
        super(_OVSOvfSet, self).initializeFromPath(self.path, 'r')
        return super(_OVSOvfSet, self).verifyManifest()

    def get_file_list(self):
        '''
        Get a list of files in ova package.
        '''
        return [ti.name for ti in self.tf.getmembers()]
        

    def fetch_files(self, files, target_dir):
        '''
        Fetch files to target directory.

        @param files: file list which need fetching
        @type files: list of Strings
        
        @param target_dir: store the fetched files here 
        @type target_dir: string
        
        @raise exception: 
        '''
        for file in files:
            self.tf.extract(file, target_dir)
       
class ErrorHandler:
    """Schema validation error/warning messages handler"""

    def __init__(self):
        """Initializes an L{ErrorHandler} object."""

        self.error_list = []
        self.warning_list = []

    def error(self, msg, data=None):
        """Method used to store error messages

        @type msg: string
        @param msg: Error message to store.
        @type data: None
        @param data: Ignored parameter.
        """
        self.error_list.append(msg)

    def warning(self, msg, data=None):
        """Method used to store warning messages

        @type msg: string
        @param msg: Warning message to store.
        @type data: None
        @param data: Ignored parameter.
        """
        self.warning_list.append(msg)

