#!/usr/bin/env python
#^#
# OVSRemasterServer.py
#
#
# Copyright (C) 2006, 2007 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.
#$#

# dlmfs document
#http://git.kernel.org/?p=linux/kernel/git/torvalds/linux-2.6.git;a=blob;f=Documentation/filesystems/dlmfs.txt

import socket
import re
import os
import logging
import logging.handlers
from time import sleep
from os.path import exists, join, basename

from OVSDaemon import daemonize
from OVSConfig import init_config, get_config
from OVSConstants import *
from OVSCommons import *
from OVSCDS import *
from OVSMultiprocess import *
from OVSXMasterFailover import *

#setup the logging system, no dependency to the log server.
formatter = logging.Formatter(fmt="%(asctime)s %(levelname)s=> %(message)s", datefmt="%Y-%m-%d %H:%M:%S")
handler = logging.handlers.RotatingFileHandler(join(LOG_DIR, "ovs_remaster.log"), "a", 1024*1024, 3)
handler.setFormatter(formatter)
logging.getLogger("").setLevel(logging.DEBUG)
logging.getLogger("").addHandler(handler)

def run():
    """
    NOTE:
    acquire_master_lock:
        holder = db_load("cluster", "master_vip_holder")
        if vip is pingable:
            if not vip_holder is me:
                release_master_lock()
        else:
            master = next_master()
            master.setup_master_vip()
            release_master_lock()
    """
    logging.info("---------------- REMASTER SERVER STARTED----------------")
    take_crown = True
    while True:
        try:
            #init_config() #let the config modification take effect immediate
            config = get_config()
            server_remaster_interval = int(config.get("ha", "server.remaster.interval"))

            #check 1) pool ha, 2)master vip
            pool_ha_enable = db_load("cluster", "pool_ha_enable", get=True)
            master_vip = db_load("cluster", "master_vip", get=True)

            if pool_ha_enable and master_vip and acquire_master_dlm_lock():
                #everything is good :=
                #1) host_alive(vip)
                #2) i_hold_the_vip(vip)
                #3) i_am_the_master()
                if host_alive(master_vip):
                    if i_hold_the_vip(master_vip):
                        if i_am_the_master():
                            #everything is good
                            if take_crown: #coronation
                                take_crown = False
                                logging.info("run(): i am the new master. vip=%s" % master_vip)
                            #else: #routine check
                            #    pass
                        else:
                            take_crown = True
                            logging.info("run(): release_master_vip ...")
                            ifconfig_vip_down(master_vip)
                            logging.info("run(): release_master_dlm_lock ...")
                            release_master_dlm_lock()
                    else:
                        take_crown = True
                        logging.info("run(): release_master_dlm_lock ...")
                        release_master_dlm_lock()
                else:
                    take_crown = True
                    try:
                        #GETVAL() will raise exception if operation failed.
                        rs = cluster_get_next_master()
                        logging.info("run(): cluster_get_next_master: => %s" % rs)
                        hostname = GETVAL(rs)
                        hostname = hostname.encode("utf-8")
                        srv = db_load("srv", hostname)
                        master = ServerProxy(srv["agt_url"])
                        rs = master.clusterm_setup_master_env()
                        logging.info("run(): clusterm_setup_master_env: => %s" % rs)
                        rs = GETVAL(rs)
                    finally:
                        if not i_am_the_master():
                            logging.info("run(): release_master_dlm_lock ...")
                            release_master_dlm_lock()

        except Exception, e:
            errmsg = fmt_err_msg(e)
            logging.error("run(): => %s" % errmsg)

        #sleep for next cycle
        sleep(server_remaster_interval)

def main():
    pid_filename = basename(__file__).split(".")[0]
    daemonize(pid_filename)
    run()

if __name__ == "__main__":
    main()

