#!/usr/bin/env python
#^#
# OVSServices.py
#
# <description>
#
#
# Copyright (C) 2006, 2007 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.
#$#

import os
import imp

import OVSSiteRMServer  #resource manager -- server part #pylint:    disable-msg=W0611
import OVSSiteRMVM      #resource manager -- vm part #pylint:    disable-msg=W0611

import OVSSiteMonitor   #pylint:    disable-msg=W0611
import OVSXMonitor      #sys_    #pylint:    disable-msg=W0611

import OVSSiteSysAdmin  #pylint:    disable-msg=W0611
import OVSXSysAdmin     #sys_    #pylint:    disable-msg=W0611

import OVSSiteUtility   #pylint:    disable-msg=W0611
import OVSXUtility      #utl_ & xen_ [for create/remove disk]    #pylint:    disable-msg=W0611

import OVSSiteVM        #pylint:    disable-msg=W0611
import OVSXXenVM        #xen_    #pylint:    disable-msg=W0611

import OVSSiteVMConfig  #pylint:    disable-msg=W0611
import OVSXXenVMConfig  #xen_    #pylint:    disable-msg=W0611

import OVSSiteVMInstall #pylint:    disable-msg=W0611
import OVSXXenVMInstall #xen_    #pylint:    disable-msg=W0611

import OVSSiteXenStore  #pylint:    disable-msg=W0611
import OVSXXenStore     #xen_    #pylint:    disable-msg=W0611

import OVSSiteSR        #pylint:    disable-msg=W0611

import OVSSiteHA        #pylint:    disable-msg=W0611
import OVSXHA               #ha_    #pylint:    disable-msg=W0611
import OVSXMasterFailover   #cluster(m)_    #pylint:    disable-msg=W0611

import OVSSiteCluster   #cluster_    #pylint:    disable-msg=W0611
import OVSXCluster      #clusterm_    #pylint:    disable-msg=W0611

import OVSXSysInfo      #get_        #pylint:    disable-msg=W0611
import OVSXOVF          #ovf_        #pylint:    disable-msg=W0611

import OVSAsyncTaskCore             #pylint:    disable-msg=W0611
import OVSAsyncTaskAux              #pylint:    disable-msg=W0611
import OVSAsyncTaskSupport          #pylint:    disable-msg=W0611

import OVSV2V                       #pylint:    disable-msg=W0611
import OVSClone                     #pylint:    disable-msg=W0611
import OVSVMXParser                 #pylint:    disable-msg=W0611
import OVSVMDKParser                #pylint:    disable-msg=W0611

from OVSWrappers import D
from OVSCommons import exposed

#----------------------------------------------------------
#special function for test xmlrpc call
def echo(msg):
    return "success:echo=%s" % msg

def sleep(secs):
    import time
    secs = float(secs)
    time.sleep(secs)
    return "success:sleep=%f" % secs

#----------------------------------------------------------
from OVSAsyncTaskCore import init_asyncenv, AsyncProxy
from inspect import ismodule
class OVSServices(object):
    def __init__(self):
        #ovs-agent
        init_asyncenv()
        self.async_proxy = AsyncProxy()

        modules = [ eval(attr) for attr in globals() if ismodule(eval(attr)) ]
        for module in modules:
            for name in dir(module):
                _attr = getattr(module, name)
                if callable(_attr):
                    # Only support a method once
                    if not hasattr(self, name):
                        if (hasattr(_attr, "exposed") and _attr.exposed == True) or (
                                hasattr(_attr, "interconnect") and _attr.interconnect == True):
                            # D('%s.__module__: %s' % (_attr, _attr.__module__))
                            # _attr.orig_func.__doc__ = "Module: %s\n%s" % (_attr.orig_func.__module__, _attr.orig_func.__doc__)
                            if name != _attr.__name__:
                                _attr.__doc__ = "Module: %s. Alias of %s or itself.\n%s" % (
                                    _attr.__module__, _attr.__name__, _attr.__doc__)
                            else:
                                _attr.__doc__ = "Module: %s\n%s" % (_attr.__module__, _attr.__doc__)
                            setattr(self, name, _attr)

        self.echo = echo
        self.sleep = sleep

        self._load_modules()

#        #xenapi
#        import xen.xend.XendAPI as XendAPI
#        import OVSXendAuthSessions
#        XendAPI.auth_manager = OVSXendAuthSessions.instance
#        self.xenapi = XendAPI.XendAPI(XendAPI.AUTH_NONE)
#
#        for meth_name in dir(self.xenapi):
#            if not meth_name.startswith("_"):
#                meth = getattr(self.xenapi, meth_name)
#                if callable(meth) and hasattr(meth, 'api'):
#                    setattr(self, getattr(meth, 'api'), meth)
#
#        from xen.xend.XendAPI import XendAPIAsyncProxy
#        self.async_xenapi = XendAPIAsyncProxy(self.xenapi)
#
    def _load_modules(self):
        '''load the modules'''
        CUR_DIR = os.path.abspath(os.path.dirname(__file__))
        module_dir = os.path.join(CUR_DIR, 'modules')

        filenames = []
        for fn in os.listdir(module_dir):
            if fn.endswith('.py') and not fn.startswith('_'):
                filenames.append(os.path.join(module_dir, fn))

        for filename in filenames:
            module_name = os.path.basename(filename)[:-3]
            # D('loading %s...' % module_name)
            try: module = imp.load_source(module_name, filename)
            except Exception, e:
                raise e
            else:
                if hasattr(module, '_init_'):
                    _init_ = getattr(module, '_init_')
                    if callable(_init_):
                        _init_()
                for name in dir(module):
                    if not name.startswith('_'):
                        attr = getattr(module, name)
                        # Only export a method once
                        if not hasattr(self, name):
                            # Only export functions defined in current file
                            if callable(attr) and attr.__module__ == module_name:
                                # D('%s.__doc__: %s' % (attr, attr.__doc__))
                                # D('%s.__module__: %s' % (attr, attr.__module__))
                                attr.__doc__ = "Module: %s\n%s" % (attr.__module__, attr.__doc__)
                                setattr(self, name, exposed(attr))

#    def _dispatch(self, method, params):
#        from xen.xend.XendAPI import XendAPIAsyncProxy  #pylint:    disable-msg=F0401
#        if not method.startswith(XendAPIAsyncProxy.method_prefix) \
#                and not method.startswith(AsyncProxy.async_method_prefix):
#            if hasattr(self, method):
#                _method = getattr(self, method)
#                rs = _method(*params)
#                return rs
#            else:
#                raise Exception("method: <%s> not supported!" % method)
#        elif method.startswith(AsyncProxy.async_method_prefix):
#            return self.async_proxy._dispatch(method, params)
#        elif method.startswith(XendAPIAsyncProxy.method_prefix):
#            return self.async_xenapi._dispatch(method, params)
#        else:
#            raise Exception("method: <%s> not supported!" % method)

    def _dispatch(self, method, params):
        if method.startswith(AsyncProxy.async_method_prefix):
            return self.async_proxy._dispatch(method, params)
        else:
            if hasattr(self, method):
                _method = getattr(self, method)
                rs = _method(*params)
                return rs
            else:
                raise Exception("Method <%s> is not supported." % method)
