#!/usr/bin/env python

import logging

from _storage import *
from OVSCommons import *

#------------------------------------------------------------------------------
@exposed
def sp_create(storage_provider, *init_args):
    logger = logging.getLogger("ovs.operation")
    try:
        sp = OVSSPManager.get_storage_provider(storage_provider, *init_args)
        uuid = sp.uuid
        db_save("LOCAL_DB,sp", uuid, sp)
        db_save("sp", uuid, sp)

        rs = SUCC({"uuid": uuid})
        logger.info("sp_create: storage_provider=%s => %s" % (
                storage_provider, rs))
        return rs
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("sp_create: error. storage_provider=%s =>%s" % (
                storage_provider, errmsg))
        return FAIL(exception=e)

@exposed
def sp_do(sp_uuid, operation, *param):
    """
    check OVSFileSP and OVSPhySP for supported operations.
    """
    logger = logging.getLogger("ovs.operation")
    try:
        try:
            sp = db_load("sp", sp_uuid)
        except:
            sp = db_load("LOCAL_DB,sp", sp_uuid)
        rs = getattr(sp, operation, *param)

        rs = SUCC(rs)
        logger.info("sp_do: sp_uuid=%s operation=%s param=(%s) => %s" % (
                sp_uuid, operation, repr(param), rs))
        return rs
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("sp_do: error. sp_uuid=%s operation=%s param=(%s) => %s" % (
                sp_uuid, operation, repr(param), errmsg))
        return FAIL(exception=e)

#------------------------------------------------------------------------------
@exposed
def sr_create(sp_uuid, name_label="Untitled", name_description="", content_type="", storage_max=-1):
    """
    @param sp_uuid: storage provider
    @param content_type: the format of vdi stored in this sr.
    @return success:uuid=<uuid>
    """
    logger = logging.getLogger("ovs.operation")
    try:
        try:
            sp = db_load("sp", sp_uuid)
        except:
            sp = db_load("LOCAL_DB,sp", sp_uuid)

        sr = OVSSR.create(sp, name_label, name_description, content_type, storage_max)
        uuid = format_uuid(sr.uuid)
        db_save("LOCAL_DB,sr", uuid, sr)
        db_save("sr", uuid, sr)

        rs = SUCC({"uuid": uuid})
        logger.info("sr_create: sp_uuid=%s, => %s" % (sp_uuid, rs))
        return rs
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("sr_create: error. storage_provider=%s =>%s" % (
                sp_uuid, errmsg))
        return FAIL(exception=e)

@exposed
def sr_do(sr_uuid, operation, *param):
    logger = logging.getLogger("ovs.operation")
    try:
        try:
            sr = db_load("sr", sr_uuid)
        except:
            sr = db_load("LOCAL_DB,sr", sr_uuid)

        rs = getattr(sr, operation)(*param)
        rs = SUCC(rs)
        logger.info("sr_do: sr_uuid=%s operation=%s param=(%s)=> %s" % (
                sr_uuid, operation, repr(param), rs))
        return rs
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("sr_do: error. sr_uuid=%s operation=%s param=(%s) =>%s" % (
                sr_uuid, operation, repr(param), errmsg))
        return FAIL(exception=e)

@exposed
def sr_set_root(sr_uuid):
    logger = logging.getLogger("ovs.operation")
    try:
        sr_orig = db_load("LOCAL_DB,node", "cluster_root_sr_uuid", get=True)
        if sr_orig:
            sr_uuid_orig = sr_orig._uuid
        else:
            sr_uuid_orig = ""

        db_save("LOCAL_DB,cluster", "cluster_root_sr_uuid_new", sr_uuid)
        db_save("cluster", "cluster_root_sr_uuid_new", sr_uuid)

        if sr_uuid != sr_uuid_orig:
            db_save("LOCAL_DB,cluster", "init_root", True)
            db_save("cluster", "init_root", True)

        try:
            sr = db_load("sr", sr_uuid)
        except:
            sr = db_load("LOCAL_DB,sr", sr_uuid)

        rs = SUCC({"fs_spec": sr.sp.fs_spec})
        logger.info("sr_set_root: success. sr_uuid=%s => %s" % (sr_uuid, rs))
        return rs
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("sr_set_root: faild. sr_uuid=%s => %s" % (sr_uuid, errmsg))
        return FAIL(exception=e)


@exposed
def sr_umount(sr_uuid):
    logger = logging.getLogger("ovs.operation")
    try:
        sr = db_load("sr", sr_uuid)
        sr.sp.umount()

        logger.info("sr_umount: success. sr_uuid=%s" % sr_uuid)
        return SUCC()
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("sr_umount: failed. sr_uuid=%s => %s" % (sr_uuid, errmsg))
        return FAIL(exception=e)


from OVSMultiprocess import *
@exposed
def sr_delete(sr_uuid):
    logger = logging.getLogger("ovs.operation")
    try:
        exclude = get_server_list_with_conn_err()
        operation = "sr_umount"
        param = (sr_uuid,)
        servers_do_wrapper(operation, param, logger, select_type="db-l", server_list=exclude)

        sr = db_load("sr", sr_uuid)
        sp_uuid = sr.sp._uuid

        db_remove("sr", sr_uuid)
        db_remove("sp", sp_uuid)

        cluster_root_sr_uuid_new = db_load("cluster", "cluster_root_sr_uuid_new", get=True)
        if sr_uuid == cluster_root_sr_uuid_new:
            db_remove("cluster", "cluster_root_sr_uuid_new")

        logger.info("sr_delete: success. sr_uuid=%s" % sr_uuid)
        return SUCC()
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("sr_delete: failed. sr_uuid=%s => %s" % (sr_uuid, errmsg))
        return FAIL(exception=e)
