#!/usr/bin/env python
#^#
# OVSSiteSysAdmin.py
#
# <description>
#
#
# Copyright (C) 2006, 2007 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.
#$#

import logging
from OVSServerProxy import ServerProxy
from OVSDB import db_get_srv
from OVSCommons import exposed, fmt_err_msg

@exposed
def shutdown_server(hostname):
    """
    shutdown the specified server

    @return	    "success:<hostname>" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.root")
    try:
	srv = db_get_srv(hostname)
	sp = ServerProxy(srv['agt_url'])
	rs = sp.sys_shutdown()
	if not rs.startswith('success'):
            raise Exception(rs)

        logger.info("shutdown_server: success. %s => %s" % (hostname, rs))
        return "success:%s" % hostname
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("shutdown_server: failed. %s => %s" % (hostname, errmsg))
	return "failed:%s" % errmsg

@exposed
def reboot_server(hostname):
    """
    reboot the specified server

    @return	    "success:<hostname>" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.root")
    try:
	srv = db_get_srv(hostname)
	sp = ServerProxy(srv['agt_url'])
	rs = sp.sys_reboot()
	if not rs.startswith('success'):
            raise Exception(rs)

        logger.info("reboot_server: success. %s => %s" % (hostname, rs))
        return "success:%s" % hostname
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("reboot_server: failed. %s => %s" % (hostname, errmsg))
	return "failed:%s" % errmsg

@exposed
def update_agent_password(hostname, password):
    """Update agent password

    @return "success" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.operation")
    try:
        srv = db_get_srv(hostname)
        if srv["agent_version"] <= (2, 2, 0, 51):
            raise Exception("sys_update_agent_password() is not supported! srv('%s') agent_version=%s." % \
                    (hostname, repr(srv["agent_version"])))
        sp = ServerProxy(srv['agt_url'])
        rs = sp.sys_update_agent_password(password)
        if not rs.startswith("success"):
            raise Exception(rs)

        #re-register this server to the master server
        from OVSSiteRMServer import register_server
        from OVSConstants import DEFAULT_AGENT_USERNAME, DEFAULT_AGENT_PORT
        register_server(hostname, srv['srv_role'], True, DEFAULT_AGENT_USERNAME, password, DEFAULT_AGENT_PORT, True)

        logger.info("update_agent_password: success. srv('%s')" % hostname)
        return "success"
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("update_agent_password: failed. srv('%s') => %s" % (hostname, errmsg))
        return "failed:%s" % errmsg

