# Copyright (C) 2009 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.
'''
Created on 2009-6-26

@author: tizhang <tiger.zhang@oracle.com>
'''

import sys
try:
    import simplejson as json
except:
    json = None

import logging

def ovs_success(result=None):
    ret = "success"
    if result != None:
        if isinstance(result, str):
            ret += ":" + result
        else:
            if json:
                ret += ":" + json.dumps(result)
            else:
                raise
    return ret

def is_success(s):
    return s.startswith('success')

def raise_if_not_success(s):
    if not s.startswith('success'):
        raise Exception(s)

def get_success_result(s):
    return s[len('success:'):]
    
def ovs_failed(result=None):
    ret = "failed"
    if result != None:
        ret += ":" + result
    return ret

def is_failed(s):
    return s.startswith('failed')

def get_failed_result(s):
    return s[len('failed:')]

def D(msg):
    caller_name = sys._getframe(1).f_code.co_name
    
    logger = logging.getLogger("ovs.debug")
    logger.debug("%s: %s" % (caller_name, msg))
    
def OI(msg):
    caller_name = sys._getframe(1).f_code.co_name
    
    logger = logging.getLogger('ovs.operation')
    logger.info('%s: %s' % (caller_name, msg))

def OE(msg):
    caller_name = sys._getframe(1).f_code.co_name
    
    logger = logging.getLogger('ovs.operation')
    logger.error('%s: %s' % (caller_name, msg))
    
if __name__ == '__main__':
    pass