#!/usr/bin/env python
#^#
# Copyright (C) 2006, 2007 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.
#$#

import re
import os
import logging
from os.path import dirname, basename, join, isdir, isfile, exists, islink, ismount
from glob import glob

from OVSXCluster import clusterm_status_heartbeat
from OVSServerProxy import *
from OVSProtocol import *
from OVSExceptionLookup import *
from OVSException import *
from OVSConstants import *
from OVSCommons import *
from OVSCDS import *
from OVSXAPIUtil import get_running_vms


@interconnect
def ha_check_exist(path):
    """
    NOTE: deprecated!!
    """
    logger = logging.getLogger("ovs.query")
    try:
        _dirname = dirname(path)
        filename = "%s%s" % (HA_TOUCH_PREFIX, basename(path))
        touch_file = join(_dirname, filename)
        if not isfile(touch_file):
            raise Exception("file touch path('%s') does not exists!" % touch_file)
        logger.info("ha_check_exist: success. file touch path('%s')." % touch_file)
        return "success"
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("ha_check_exist: failed. => %s" % errmsg)
        return "failed:%s" % errmsg


def _find_ocfs2_dev(uuid, dev_peer):
    # dev_peer: "dev/sdb1" -> ptn: "/dev/sd*1"
    v = list(dev_peer)
    idx = -1
    while v[idx] in "0123456789":
        idx -= 1
    v[idx] = "*"
    ptn = "".join(v)
    devs = glob(ptn)

    #find dev with same uuid
    tgt_dev = []
    for dev in devs:
        try:
            _uuid = old_run_cmd("debugfs.ocfs2 %s <<<stats 2>/dev/null | awk '/UUID/ {print $2;}'" % dev).splitlines()[-1]
            if _uuid == uuid:
                tgt_dev.append(dev)
        except:
            pass

    return tgt_dev


def get_ocfs2_nodes():
    n_path = "/sys/kernel/config/cluster/ocfs2/node"
    if exists(n_path):
        return os.listdir(n_path)
    else:
        return []

def check_ocfs2_nodes(master_nodes):
    m_nodes = set(master_nodes)
    s_nodes = set(get_ocfs2_nodes())
    return s_nodes.issubset(m_nodes)


@exposed
def check_shared_storage_mount(marshalled_shared_fs_line, marshalled_shared_repositories, shared_fs_extra, master_nodes):
    logger = logging.getLogger("ovs.operation")
    try:
        #base shared storage
        shared_fs, shared_fs_type, shared_fs_line = parse_shared_fs()

        master_shared_fs_line = unmarshall_string(marshalled_shared_fs_line)
        master_shared_fs, master_mount_point, master_shared_fs_type, master_mount_option = master_shared_fs_line.split()[0:4]

        if master_shared_fs_type == "ocfs2":
            if not check_ocfs2_nodes(master_nodes):
                raise Exception("Cluster is incorrectly configured on the virtual server.")

        if shared_fs and shared_fs_type == "ocfs2" and master_shared_fs_type == "ocfs2":
            #already mounted and is ocfs2 partition mounted by device name
            uuid = parse_result("success:%s" % shared_fs_extra)["uuid"]
            tgt_dev = _find_ocfs2_dev(uuid, master_shared_fs)
            if shared_fs not in tgt_dev:
                raise Exception("%s already mounted but is not the same as master node." % OVS_BASE)
        elif shared_fs and (master_shared_fs_type != shared_fs_type or master_shared_fs != shared_fs):
            #already mounted, but not the same as master node
                raise Exception("%s already mounted but is not the same as master node." % OVS_BASE)
        elif not shared_fs:
            #local filesytem, try to mount the shared storage
            if master_shared_fs_type == "ocfs2" and master_shared_fs.startswith("/dev"):
                uuid = parse_result("success:%s" % shared_fs_extra)["uuid"]
                tgt_dev = _find_ocfs2_dev(uuid, master_shared_fs)
                if not tgt_dev:
                    raise Exception("cannot find the equivalent filesystem as master partition with uuid='%s'" % uuid)

        logger.info("check_shared_storage_mount: success")
        return "success"
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("check_shared_storage_mount: error. =>%s" % errmsg)
        return "failed:%s" % errmsg


@exposed
def ha_mount_shared_storage(marshalled_shared_fs_line, marshalled_shared_repositories, shared_fs_extra=""):
    logger = logging.getLogger("ovs.operation")
    try:
        #base shared storage
        shared_fs, shared_fs_type, shared_fs_line = parse_shared_fs()

        master_shared_fs_line = unmarshall_string(marshalled_shared_fs_line)
        master_shared_fs, master_mount_point, master_shared_fs_type, master_mount_option = master_shared_fs_line.split()[0:4]

        if shared_fs and shared_fs_type == "ocfs2" and master_shared_fs_type == "ocfs2":
            #already mounted and is ocfs2 partition mounted by device name
            uuid = parse_result("success:%s" % shared_fs_extra)["uuid"]
            tgt_dev = _find_ocfs2_dev(uuid, master_shared_fs)
            if shared_fs not in tgt_dev:
                raise Exception("%s already mounted but is not the same as master node." % OVS_BASE)
        elif shared_fs and (master_shared_fs_type != shared_fs_type or master_shared_fs != shared_fs):
            #already mounted, but not the same as master node
                raise Exception("%s already mounted but is not the same as master node." % OVS_BASE)
        elif not shared_fs:
            #local filesytem, try to mount the shared storage
            if master_shared_fs_type == "ocfs2" and master_shared_fs.startswith("/dev"):
                uuid = parse_result("success:%s" % shared_fs_extra)["uuid"]
                tgt_dev = _find_ocfs2_dev(uuid, master_shared_fs)
                if tgt_dev:
                    shared_fs = tgt_dev[0]
                else:
                    raise Exception("cannot find the equivalent filesystem as master partition with uuid='%s'" % uuid)
            else:
                shared_fs = master_shared_fs
            #cmd = "mount -t %s -o %s %s %s" % (master_shared_fs_type, master_mount_option, shared_fs, master_mount_point)
            cmdlist = ["mount", "-t", master_shared_fs_type, "-o", master_mount_option, shared_fs, master_mount_point]
            run_cmd(args=cmdlist)

        #extended shared storage
        shared_repositories = unmarshall_string(marshalled_shared_repositories)
        f = open("/etc/ovs/repositories", "w")
        try:
            f.write(shared_repositories)
        finally:
            f.close()
        #cmd = "service ovsrepositories start"
        cmdlist = ["service", "ovsrepositories", "start"]
        run_cmd(args=cmdlist)

        logger.info("ha_mount_shared_storage: success")
        return "success"
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("ha_mount_shared_storage: error. =>%s" % errmsg)
        return "failed:%s" % errmsg

@exposed
def ha_setup_ocfs2_cluster(marshalled_cluster_config, shared_fs_type="ocfs2"):
    """
    *) reconfigure the o2cb service
    *) start the heartbeating
        /OVS is ocfs2, mount the partition
        /OVS is nfs, start heartbeat against the dmnfs heartbeat device
    """
    logger = logging.getLogger("ovs.operation")
    try:
        #configure & start cluster service
        if not isdir("/etc/ocfs2"):
            os.makedirs("/etc/ocfs2")
        f = open("/etc/ocfs2/cluster.conf", "w")
        try:
            f.write(unmarshall_string(marshalled_cluster_config))
        finally:
            f.close()

        logger.info("ha_setup_ocfs2_cluster: trying to reconfigure the cluster...")
        cmd = join(dirname(__file__), "utils/config_o2cb.sh")
        cmdlist = [cmd]
        run_cmd(args=cmdlist)

        #start heartbeating
        if shared_fs_type == "ocfs2":
            #mount will let the server's start hearting, see ha_sync_shared_storage_mount()
            pass
        elif shared_fs_type == "nfs":
            #cmd = "/usr/lib/ovs/ovs-cluster-check --alter-fstab"
            cmdlist = ["/usr/lib/ovs/ovs-cluster-check", "--alter-fstab"]
            run_cmd(args=cmdlist)
        else:
            raise Exception("filesystem type '%s' not in [ocfs2, nfs]." % shared_fs_type)

        logger.info("ha_setup_ocfs2_cluster: success.")
        return "success"
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("ha_setup_ocfs2_cluster: error. =>%s" % errmsg)
        return "failed:%s" % errmsg


@exposed
def ha_check_cluster_heartbeat():
    """
    check wheter the ocfs2 cluster heartbeat is alive
    """
    logger = logging.getLogger("ovs.query")
    try:
        rs = "failed"
        shared_fs, shared_fs_type, shared_fs_line = parse_shared_fs()
        if shared_fs_type == "ocfs2":
            #cmd = "ocfs2_hb_ctl -I -d %s" % shared_fs
            cmdlist = ["ocfs2_hb_ctl", "-I", "-d", shared_fs]
            l = run_cmd(args=cmdlist).splitlines()
            if l[-1].split()[1] != "0":
                rs = "success"
        elif shared_fs_type == "nfs":
            #cmd = "dmsetup table --target=nfs"
            cmdlist = ["dmsetup", "table", "--target=nfs"]
            l = run_cmd(args=cmdlist).splitlines()
            for line in l:
                parts = line.split()
                if len(parts) > 4 and parts[4] == HA_NFS_HEARTBEAT_FILE:
                    dev = parts[0].split(":")[0]
                    #cmd = "ocfs2_hb_ctl -I -d /dev/mapper/%s" % dev
                    cmdlist = ["ocfs2_hb_ctl", "-I", "-d", "/dev/mapper/%s" % dev]
                    _l = run_cmd(args=cmdlist).splitlines()
                    if _l[-1].split()[1] != "0":
                        rs = "success"
                        break
        else:
            raise Exception("only [ocfs2, nfs] shared filesystem is supported.")

        logger.info("ha_check_cluster_heartbeat: shared_fs='%s' %s." % (shared_fs, rs))
        return rs
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("ha_check_cluster_heartbeat: error. %s" % errmsg)
        return "failed: %s" % errmsg


@exposed
def ha_join_dlm_domain(domain_name="ovm", keep_silence=False):
    logger = logging.getLogger("ovs.operation")
    try:
        _dir = join(DLM, basename(domain_name))
        if not isdir(_dir):
            rs = clusterm_status_heartbeat(keep_silence=True)
            status = GETVAL(rs)
            if status != HEARTBEAT_STATUS_ACTIVE:
                raise OVSException(ERR_NO_HEARTBEAT)
            os.mkdir(_dir)
        #else: already joined

        rs = "success"
        if not keep_silence:
            logger.info("ha_join_dlm_domain: =>%s" % rs)
        return rs
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("ha_join_dlm_domain: domain('%s') error. => %s" % (domain_name, errmsg))
        return "failed:%s" % errmsg

@exposed
def register_dlm_lock(lock_name, uuid, hostname):
    """
    register dlm lock
    """
    logger = logging.getLogger("ovs.operation")
    try:
        cds = CDS("dlm")
        try:
            dlm = cds.db_dlm
            dlm[lock_name] = {
                    "uuid": uuid,
                    "hostname": hostname
                    }
        finally:
            cds.close()

        logger.info("register_dlm_lock: success. lock_name('%s'), uuid('%s'), hostname('%s')" % (lock_name, uuid, hostname))
        return "success"
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("register_dlm_lock: failed.  lock_name('%s'), uuid('%s'), hostname('%s')() => %s" % (lock_name, uuid, hostname, errmsg))
        return "failed:%s" % errmsg

@exposed
def unregister_dlm_lock(lock_name):
    """
    unregister dlm lock
    """
    logger = logging.getLogger("ovs.operation")
    try:
        cds = CDS("dlm")
        try:
            dlm = cds.db_dlm
            if lock_name in dlm:
                del dlm[lock_name]
        finally:
            cds.close()

        logger.info("unregister_dlm_lock: success. lock_name('%s')" % lock_name)
        return "success"
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("unregister_dlm_lock: failed. lock_name('%s') => %s" (lock_name, errmsg))
        return "failed:%s" % errmsg

def ha_register_lock(uuid, lock_name):
    logger = logging.getLogger("ovs.operation")
    try:
        d_cluster = db_dump("cluster")
        sp = ServerProxy(d_cluster["master_agt_url"])
        d_node = db_dump("LOCAL_DB,node")
        hostname = d_node["registered_hostname"]

        rs = sp.register_dlm_lock(lock_name, uuid, hostname)
        if not rs.startswith("success"): raise Exception(rs)

        logger.info("ha_register_lock: success. uuid('%s'), lock_name('%s')" % (uuid, lock_name))
        return rs

    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("ha_register_lock: failed. uuid('%s'), lock_name('%s') => %s" % (uuid, lock_name, errmsg))
        return "failed:%s" % errmsg

def ha_unregister_lock(lock_name):
    logger = logging.getLogger("ovs.operation")
    try:
        d_cluster = db_dump("cluster")
        sp = ServerProxy(d_cluster["master_agt_url"])
        rs = sp.unregister_dlm_lock(lock_name)
        if not rs.startswith("success"): raise Exception(rs)

        logger.info("ha_unregister_lock: success. lock_name('%s')" % lock_name)
        return rs

    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("ha_unregister_lock: failed. lock_name('%s') => %s" % (lock_name, errmsg))
        return "failed:%s" % errmsg

@exposed
def ha_set_dlm_lock(lock, name=""):
    """
    @param lock: uuid of the vm
    @param name: name of the vm
    @return "success": if the lock successfully or "failed" with the error message
    """
    logger = logging.getLogger("ovs.operation")
    try:
        if not lock.strip():
            raise Exception("lock must be specified!")

        rs = ha_join_dlm_domain(DLM_DOMAIN)
        if not rs.startswith("success"):
            raise Exception(rs)

        #limit dlm lock name to 31 chars
        lock_path = join(DLM_DOMAIN, re.sub(r"[^a-zA-Z0-9]+", "", lock)[-31:])

        if exists(lock_path):
            raise Exception("lock('%s') has already been set!" % lock_path)

        try:
            fd = os.open(lock_path, os.O_CREAT | os.O_NONBLOCK)
            os.close(fd)
        except Exception, e:
            try:
                os.remove(lock_path) #get rid of the side effect
            except:
                pass
            errmsg = fmt_err_msg(e)
            raise Exception("create lock('%s') failed. %s" % (lock_path, errmsg))

        #create the EX lock
        fd = os.open(lock_path, os.O_RDWR | os.O_NONBLOCK)
        #close it, or the agent process will hold the lock
        os.close(fd)

        rs = ha_register_lock(lock, name)
        if not rs.startswith("success"):
            raise Exception(rs)

        logger.info("ha_set_dlm_lock: lock('%s') has been set for vm('%s')" % (lock, name))
        return "success"
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("ha_set_dlm_lock:failed. lock('%s') name('%s')=> %s" % (lock, name, errmsg))
        return "failed: %s" % errmsg

ha_accquire_lock = ha_set_dlm_lock

@exposed
def ha_release_dlm_lock(lock, name=""):
    """
    @param lock: uuid of the vm
    @param name: name of the vm
    @return: "success" if the lock successfully or "failed" with the error message
    """
    logger = logging.getLogger("ovs.operation")
    try:
        if not lock.strip():
            raise Exception("lock must be specified!")

        lock_path = join(DLM_DOMAIN, re.sub(r"[^a-zA-Z0-9]+", "", lock)[-31:])
        if exists(lock_path):
            os.remove(lock_path)

        rs = ha_unregister_lock(name)
        if not rs.startswith("success"):
            raise Exception(rs)

        logger.info("ha_release_dlm_lock:lock('%s') name('%s') success." % (lock, name))
        return "success"
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("ha_release_dlm_lock:failed. lock('%s') name('%s') => %s" % (lock, name, errmsg))
        return "failed: %s" % errmsg

ha_release_lock = ha_release_dlm_lock


@exposed
def ha_start_agent_guard():
    logger = logging.getLogger("ovs.operation")
    try:
        run_cmd_bg(AGENT_GUARD_PROCESS)

        logger.info("ha_start_agent_guard: success.")
        return "success"
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("ha_start_agent_guard: error. => %s" % errmsg)
        return "failed:%s" % errmsg


@exposed
def ha_set_shutdown_mode(action="enter"):
    """
    1) tell the master the server is in shutdown mode
    2) create the shutdown mode flag
    """
    logger = logging.getLogger("ovs.operation")
    try:
        logger.info("ha_set_shutdown_mode: inform master agent, %s shutdown mode." % action)
        try:
            master_agt_url = db_load("cluster", "master_agt_url")
        except:
            master_agt_url = ""

        if not master_agt_url:
            logger.debug("ha_set_shutdown_mode: no master access information available.")
        else:
            from OVSCommons import get_my_ips
            my_ips = get_my_ips()
            my_ips = [ip for ip in my_ips if not ip.startswith("127")]
            sp = ServerProxy(master_agt_url)
            rs = "success"
            if action == "enter":
                rs = sp.set_shutdown_mode(",".join(my_ips), True)
            elif action == "leave":
                rs = sp.set_shutdown_mode(",".join(my_ips), False)

            if not rs.startswith("success"):
                raise Exception(rs)

        from OVSConstants import AGENT_SHUTDOWN_MODE_FLAG
        if action == "enter":
            logger.info("ha_set_shutdown_mode: create shutdown mode flag.")
            open(AGENT_SHUTDOWN_MODE_FLAG, "w").close()
        elif action == "leave":
            logger.info("ha_set_shutdown_mode: remove shutdown mode flag.")
            if exists(AGENT_SHUTDOWN_MODE_FLAG):
                os.remove(AGENT_SHUTDOWN_MODE_FLAG)

        return "success"
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("ha_set_shutdown_mode: failed. =>%s" % errmsg)
        return "failed: %s" % errmsg

@exposed
def ha_reset_dlm_lock(locks):
    logger = logging.getLogger("ovs.operation")
    logger.info("ha_reset_dlm_lock: start...")
    try:
        vms = get_running_vms()
        locks_name = [ k for k, v in locks ]
        vms_name = [ k for k, v in vms ]

        for k, v in vms:
            if not k in locks_name:
                ha_set_dlm_lock(v, k)

        for k, v in locks:
            if not k in vms_name:
                ha_release_dlm_lock(v, k)

        logger.info("ha_reset_dlm_lock: success. locks('%s')" % repr(locks))
        return "success"
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("ha_reset_dlm_lock: failed. locks('%s') => %s" % (repr(locks), errmsg))
        return "failed:%s" % errmsg

@exposed
def remove_dlm_lock():
    logger = logging.getLogger("ovs.operation")
    try:
        ptn = "%s/*" % DLM_DOMAIN
        for lock in glob(ptn):
            if lock != DLM_LOCK_MASTER:
                os.remove(lock)

        logger.info("remove_dlm_lock: success.")
        return "success"
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("remove_dlm_lock: failed. => %s" % errmsg)
        return "failed:%s" % errmsg

@interconnect
def ha_get_hostname_ip():
    logger = logging.getLogger("ovs.operation")
    try:
        hostname = socket.gethostname()
        ip = socket.gethostbyname(hostname)
        rs = "hostname=%s;ip=%s" % (hostname, ip)
        logger.info("ha_get_hostname_ip: %s" % rs)
        return "success:%s" % rs
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("ha_get_hostname_ip: failed. => %s" % errmsg)
        return "failed:%s" % err

@interconnect
def ha_check_ovs_base():
    logger = logging.getLogger("ovs.operation")
    try:
        if not islink(OVS_BASE) and ismount(OVS_BASE):
            raise OVSException(ERR_OVS_IS_MOUNTPOINT)

        logger.info("ha_check_ovs_mount: success.")
        return "success"
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("ha_check_ovs_mount: failed. => %s" % errmsg)
        return "failed:%s" % errmsg

def _read_value(filename):
    f = open(filename, "r")
    v = f.read().strip()
    f.close()
    return v

@interconnect
def ha_get_ocfs2_config():
    logger = logging.getLogger("ovs.operation")
    try:
        # default cluster name
        cluster_name = "ocfs2"
        # parse ocfs2 cluster conf
        if exists(OCFS2_CLUSTER_CONF):
            conf_dict = parse_ocfs2_cluster_conf(OCFS2_CLUSTER_CONF)
            if conf_dict["cluster"].get("name"):
                cluster_name = conf_dict["cluster"]["name"]

        ptn = "/sys/kernel/config/cluster/%s/node/*" % cluster_name
        l = glob(ptn)

        node_list = []
        for node_dir in l:
            ip = _read_value(join(node_dir, "ipv4_address"))
            port = _read_value(join(node_dir, "ipv4_port"))
            num  = _read_value(join(node_dir, "num"))
            node_list.append("%s:%s:%s:%s" % (cluster_name, num, ip, port))

        nodes = ",".join(node_list)
        logger.info("ha_get_ocfs2_config: success. nodes=%s" % nodes)
        return "success:nodes=%s" % nodes
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("ha_get_ocfs2_config: failed. => %s" % errmsg)
        return "failed:%s" % errmsg

@exposed
def ha_hook_vm_shutdown(vm_name, shutdown_reason):
    """
    inform the server pool master to update the manual shutdown flag
    """
    logger = logging.getLogger("ovs.operation")
    try:
        # only handle SHUTDOWN_poweroff
        if shutdown_reason == '0':
            try:
                master_agt_url = db_load("cluster", "master_agt_url")
            except:
                master_agt_url = ""

            if not master_agt_url:
                logger.debug("ha_hook_vm_shutdown: no master access information available.")
            else:
                sp = ServerProxy(master_agt_url)
                rs = sp.update_manual_shutdown(vm_name)
                if not rs.startswith("success"):
                    raise Exception(rs)

        logger.info("ha_hook_vm_shutdown: success. vm_name='%s', shutdown_reason='%s'" % (vm_name, shutdown_reason))
        return "success"
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("ha_hook_vm_shutdown: failed. =>%s" % errmsg)
        return "failed: %s" % errmsg
