#!/usr/bin/env python

#^#
# OVSXOVF.py
#
# OVF Support
#
#
# Copyright (C) 2006, 2007 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.
#$#

import logging
from os.path import isdir, isfile, exists, join, basename
from OVSCommons import exposed, fmt_err_msg, run_cmd
from glob import glob

@exposed
def ovf_get_vm_env_file(vm_path):
    """Get content of ovf-env.xml,
    if ovf-env.xml does not exist, create it.

    @return "success:<content of ovf-env.xml>"
    """
    logger = logging.getLogger("ovs.operation")
    try:
        env_file = join(vm_path, "ovf-env.xml")
        if not exists(env_file):
            l = glob("%s/*.ovf" % vm_path)
            if not l:
                raise Exception("no ovf file found.")
            ovf_file = join(vm_path, "%s.ovf" % basename(vm_path))
            if  ovf_file not in l:
                ovf_file = l[0]

            #generate ovf-env.xml
            # cmd = "ova --environment -n -z -f %s -o %s" % \
            #     (ovf_file, env_file)
            # run_cmd(cmd)
            run_cmd(args=['ova',
                          '--environment',
                          '-n',
                          '-z',
                          '-f', ovf_file,
                          '-o', env_file,
                          ])

            #create env iso file
            iso_file = join(vm_path, "ovf-env.iso")
            # cmd = "ova --vdisk -f %s -o %s" % (env_file, iso_file)
            # run_cmd(cmd)
            run_cmd(args=['ova',
                          '--vdisk',
                          '-f', env_file,
                          '-o', iso_file,
                          ])

            #regenerate vm.cfg
            vm_cfg_file = join(vm_path, "vm.cfg")
            # cmd = "/usr/bin/ova --xmconfig -f %s -t pvm -e %s -o %s" % \
            #         (ovf_file, vm_path, vm_cfg_file)
            # run_cmd(cmd)
            run_cmd(args=['/usr/bin/ova',
                          '--xmconfig',
                          '-f', ovf_file,
                          '-t', 'pvm',
                          '-e', vm_path,
                          '-o', vm_cfg_file,
                          ])

        f = open(env_file)
        try:
            rs = "success:%s" % f.read()
        finally:
            f.close()

        logger.info("ovf_get_vm_env_file: success. vm('%s')" % vm_path)
        return rs
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("ovf_get_vm_env_file: failed. vm('%s') => %s" % (vm_path, errmsg))
        return "failed:%s" % errmsg

@exposed
def ovf_set_vm_env_file(vm_path, content, vm_type="pvm"):
    """Write content to ovf-env.xml
    1) write ovf-env.xml
    2) recreate ov-env.iso
    3) recreate vm.cfg

    @param  vm_type in [pvm, hvm]
    @return "success"
    """
    logger = logging.getLogger("ovs.operation")
    try:
        env_file = join(vm_path, "ovf-env.xml")

        f = open(env_file, "w")
        try:
            f.write(content)
        finally:
            f.close()

        #create env iso file
        iso_file = join(vm_path, "ovf-env.iso")
        # cmd = "ova --vdisk -f %s -o %s" % (env_file, iso_file)
        # run_cmd(cmd)
        run_cmd(args=['ova',
                      '--vdisk',
                      '-f', env_file,
                      '-o', iso_file,
                      ])

        #regenerate vm.cfg
        vm_cfg_file = join(vm_path, "vm.cfg")
        # cmd = "ova --xmconfig -f %s -t %s -e %s -o %s" % \
        #         (env_file, vm_type, vm_path, vm_cfg_file)
        # run_cmd(cmd)
        run_cmd(args=['ova',
                      '--xmconfig',
                      '-f', env_file,
                      '-t', vm_type,
                      '-e', vm_path,
                      '-o', vm_cfg_file,
                      ])

        #TODO:
        #if the vm is running, reattach the iso

        logger.info("ovf_set_vm_env_file: success. vm('%s')" % vm_path)
        return "success"
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("ovf_set_vm_env_file: failed. vm('%s') => %s" % (vm_path, errmsg))
        return "failed:%s" % errmsg

