#!/usr/bin/env python
#^#
# OVSXSysAdmin.py
#
# <description>
#
#
# Copyright (C) 2006, 2007 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.
#$#

import os
import logging
from time import sleep
from signal import SIGUSR1
from OVSTimer import Timer
from OVSConstants import SHUTDOWN_FLAG
from OVSCommons import exposed, timeout_command, fmt_err_msg

@exposed
def sys_shutdown():
    """
    shutdown the server
    """
    logger = logging.getLogger("")
    try:
	#shutdown all running vms
	try:
	    timeout_command("xm shutdown -a")
	    timer = Timer(120, 10)
	    while timer.wait():
		stdout = timeout_command("xm list")
		lines = stdout.splitlines()
		if len(lines) < 3: break
	except:
            pass

        #signal the guard process to release the nowayout lock
        try:
            cmd = "pkill -%d -f OVSAgentGuard.py" % SIGUSR1
            #may not be catched by the signal handler
            for x in range(5):
                run_cmd(cmd)
            sleep(1)
        except:
            pass

        #accept no more connection
	agent_shutdown_flag = "%s.%s" % (SHUTDOWN_FLAG, os.environ.get("USER"))
        open(agent_shutdown_flag, "w").close()

	#shutdown the server
	cmd = "shutdown -h 1 &"
	rs = os.system(cmd)
	if rs:
	    raise Exception("error when processing shutdown")

	logger.info("sys_shutdown: success.")
	return "success"
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("sys_shutdown: failed. => %s" % errmsg)
	return "failed:%s" % errmsg

@exposed
def sys_reboot():
    """
    reboot the server
    """
    logger = logging.getLogger("")
    try:
	#shutdown all running vms
	try:
	    timeout_command("xm shutdown -a")
	    timer = Timer(120, 10)
	    while timer.wait():
		stdout = timeout_command("xm list")
		lines = stdout.splitlines()
		if len(lines) < 3: break
	except: pass

	#accept no more connection
	agent_shutdown_flag = "%s.%s" % (SHUTDOWN_FLAG, os.environ.get("USER"))
        open(agent_shutdown_flag, "w").close()

	#reboot the server
	cmd = "shutdown -r 1 &"
	rs = os.system(cmd)
	if rs:
	    raise Exception("error when processing reboot")

	logger.info("sys_reboot: success.")
	return "success"
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("sys_reboot: failed. => %s" % errmsg)
	return "failed:%s" % errmsg

@exposed
def sys_update_agent_password(password):
    """Update agent password

    @return "success"
    """
    logger = logging.getLogger("ovs.operation")
    try:
        import sys
        if "utils" not in sys.path:
            sys.path.insert(0, "utils")

        from utils.config_agent import update_ovsagent_user, notify_agent
        from OVSConfig import get_config
        from OVSConstants import DEFAULT_AGENT_USERNAME

        config = get_config()
        passwd_file = config.get("security", "passwdfile")
        update_ovsagent_user(passwd_file, DEFAULT_AGENT_USERNAME, password)
        notify_agent()

        logger.info("sys_update_agent_password: success.")
        return "success"
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("sys_update_agent_password: failed. => %s" % errmsg)
        return "failed:%s" % errmsg

