#!/usr/bin/env python
#^#
#
#
# Copyright (C) 2006, 2007 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.
#$#

import string
import logging
from OVSCommons import exposed, fmt_err_msg, timeout_command
from OVSCDS import db_load

@exposed
def get_oracle_vm_version():
    """
    @return oracle_vm_version=<val>

    <val> is ovs-release in '/etc/ovs-info'
    """
    logger = logging.getLogger("ovs.query")
    try:
        d = {}
        f = open("/etc/ovs-info")
        try:
            lines = f.read().splitlines()
            for line in lines:
                parts = map(string.strip, line.split(":", 1))
                if len(parts) == 2:
                    d[parts[0]] = parts[1]
        finally:
            f.close()
        if "ovs-release" not in d:
            raise Exception("cannot determine Oracle VM version through '/etc/ovs-info'.")

        rs = "success:oracle_vm_version=%s" % d['ovs-release']
        logger.info("get_oracle_vm_version: %s" % rs)
        return rs
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("get_oracle_vm_version: failed. => %s" % errmsg)
        return "failed:%s" % errmsg

@exposed
def get_rpm_version():
    """
    @return agent_version=<val>

    <val> is "rpm --query --queryformat '%{VERSION}-%{RELEASE}' ovs-agent"
    """
    logger = logging.getLogger("ovs.query")
    try:
        cmd = ['rpm', '--query', '--queryformat', '%{VERSION}-%{RELEASE}', 'ovs-agent']
        vr = timeout_command(cmd)

        rs = "success:agent_version=%s" % vr
        logger.info("get_rpm_version: %s" % rs)
        return rs
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("get_rpm_version: failed. => %s" % errmsg)
        return "failed:%s" % errmsg


_agt_ver = None

@exposed
def get_agent_version():
    """
    get version of running agent
    @return agent_version=<val>
    """
    logger = logging.getLogger("ovs.query")

    global _agt_ver
    if not _agt_ver: _agt_ver = get_rpm_version()
    rs = _agt_ver

    if rs.startswith("success"): logger.info("get_agent_version: %s" % rs)
    else: logger.error("get_agent_version: %s" % rs)

    return rs

@exposed
def get_agent_conf_mode():
    logger = logging.getLogger("ovs.query")
    try:
        flag = db_load("LOCAL_DB,node", "local_db")
        if flag:
            mode = "local"
        else:
            mode = "cluster"

        logger.info("get_agent_conf_mode: %s" % mode)
        return "success:mode=%s" % mode
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("get_agent_conf_mode: failed. => %s" % errmsg)
        return "failed:%s" % errmsg
