#!/usr/bin/python

#^#
# agtctlutil
#
# ovs agent start/stop utility
#
#
# Copyright (C) 2006, 2007 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.
#$#
import xmlrpclib
import time
import sys
import socket
import signal
import os
from os.path import basename, dirname, join, abspath
from glob import glob

from OVSConstants import PID_DIR
from OVSCommons import old_run_cmd, fmt_err_msg, run_cmd_bg, timeout_command
from OVSTimer import Timer

#-----------------------------------------------------------------------

def _get_processes_children(pid_list):
    """
    Get children of all the pids in the pid_list

    @param pid_list: a list/tuple of pid
    @type pid_list:
    """

    # FIXME: os.kill, os.killpg

    try:
        cmd = "pgrep -P %s" % ','.join(pid_list)
        children = old_run_cmd(cmd).splitlines()
    except Exception, e:
        children = []

    return children

def kill_v2v_daemon(force=False):
    l = glob('%s/v2v*.pid' % PID_DIR)
    for pid_file in l:
        try:
            f = open(pid_file, 'r')
            pid = int(f.read().strip())
            print "Terminal v2v daemon %d" % pid
            try:
                if force:
                    os.kill(pid, signal.SIGTERM)
                else:
                    os.kill(pid, signal.SIGINT)
                    print "Cancelling %d..." % pid

                    timeout = 30
                    while True:
                        try:
                            os.getpgid(pid)

                            time.sleep(1)
                            timeout -= 1
                            if timeout == 0:
                                print "Timeout"
                                os.kill(pid, signal.SIGTERM)
                                break
                        except:
                            print "OK"
                            break
            except Exception,e:
                print e
                pass
        finally:
            f.close()
        try:
            os.remove(pid_file)
        except:
            pass

#-----------------------------------------------------------------------
def kill_process_tree():
    kill_v2v_daemon(force=True)

    l = glob("%s/*.pid" % PID_DIR)
    for pid_file in l:
        process_name = basename(pid_file)[:-4] #remove ".pid"
        f = open(pid_file, "r")
        pid = f.read().strip()
        f.close()

        #recursively find all the children process of this pid
        parent_set = set([pid])
        children = _get_processes_children(parent_set)
        new_set = parent_set.union(set(children))
        while parent_set != new_set:
            parent_set = new_set
            children = _get_processes_children(list(parent_set))
            new_set = parent_set.union(set(children))

        #kill them all
        try:
            cmd = "kill -9 %s" % " ".join(list(parent_set))
            old_run_cmd(cmd)
        except:
            pass
        os.remove(pid_file)

#-----------------------------------------------------------------------
def is_reboot():
    cmd = "/sbin/runlevel"
    stdout = old_run_cmd(cmd)
    runlevel = stdout.strip().split()[1]
    if runlevel == "6":
        return True
    return False

def _get_running_vms():
    cmd = "xm list"
    try:
        stdout = timeout_command(cmd)
    except:
        return []
    lines = stdout.splitlines()
    l = [line.split()[0] for line in lines]
    l = [name for name in l if name not in ("Name", "Domain-0")]
    return l

def shutdown_running_vms():
    print "signal all the vm to shutdown..."
    cmd = "xm shutdown -a"
    old_run_cmd(cmd)

    l = _get_running_vms()
    #if cannot shutdown all the vms in 5 mins, destroy the survivors
    timeout, interval = 300, 5
    timer = Timer(timeout, interval)
    while timer.wait() and l:
        l = _get_running_vms()
        if l:
            print "vm %s still running, wait another %d seconds..." % (repr(l), interval)
    if timer.timeout():
        print "timeout (%d seconds)..." % timeout
        for vm in l:
            cmd = "xm destroy %s" % vm
            old_run_cmd(cmd)
            print "vm('%s') destroyed." % vm

    return True

#-----------------------------------------------------------------------
def do_shutdown_mode(action="enter"):
    try:
        cmd = "%s/utils/do_rpc.py ha_set_shutdown_mode %s" % (dirname(__file__), action)
        rs = old_run_cmd(cmd)
        if "success" not in rs:
            print "do_shutdown_mode(): action='%s' => %s" % (action, rs)
            return False
        return True
    except Exception, e:
        print e
        return False

def wait_clean_env(_sleep=10):
    while True:
        try:
            l = _get_running_vms()
            if not l:
                print "no vm left running."
                return True
            print "vms %s are still running! wait another %d seconds..." % (repr(l), _sleep)
            time.sleep(_sleep)
        except:
            pass

#-----------------------------------------------------------------------
ANSWER_CLEAN_ENV=0
ANSWER_SHUTDOWN_WAIT=1
ANSWER_SHUTDOWN_IMMEDIATE=2
ANSWER_CANCEL=3
def interactive_shutdown_menu():
    l = _get_running_vms()
    if not l:
        return ANSWER_CLEAN_ENV
    print "\nvms %s are still left running, what do you want the agent to do?" % repr(l)
    print "\t1) enter agent shutdown mode, migrate/shutdown them through Manager."
    print "\t2) shutdown immdediately, all the running vms will be stopped."
    print "\t3) cancel the shutdown operation."
    try:
        answer = raw_input("your choice:[3] ")
        if not answer.strip():
            answer = ANSWER_CANCEL
        else:
            answer = int(answer)
        return answer
    except KeyboardInterrupt:
        return ANSWER_CANCEL
    except:
        pass

def interactive_shutdown_mode_menu():
    l = _get_running_vms()
    if not l:
        return ANSWER_CLEAN_ENV
    print "\nvms %s are still running, what do you want the agent to do?" % repr(l)
    print "\t1) retry to check if there are vms still running. if none, shutdown."
    print "\t2) shutdown immdediately, all the running vms will be stopped."
    print "\t3) leave shutdown mode and go back to normal operation."
    try:
        answer = raw_input("your choice:[1] ")
        if not answer.strip():
            answer = ANSWER_SHUTDOWN_WAIT
        else:
            answer = int(answer)
        return answer
    except KeyboardInterrupt:
        return ANSWER_SHUTDOWN_WAIT
    except:
        pass

