#!/usr/bin/python
#============================================================================
# This library is free software; you can redistribute it and/or
# modify it under the terms of version 2.1 of the GNU Lesser General Public
# License as published by the Free Software Foundation.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#============================================================================
#

import sys

class PhySR(object):
    """ Base class for Physical based Storage Repos. """

    def __init__(self, uuid,
            storage_provider,
            name_label = 'Untitled',
            name_description = 'File Based Storage Repository',
            content_type = "LUN", # or partition?
            storage_max = -1):
        """
        @param storage_max: Maximum disk space to use in bytes. -1 for unlimited.
        """

        self.uuid = uuid
        self.type = "File Based SR"
        self.name_label = name_label
        self.name_description = name_description
        self.images = {}
        self.sp = storage_provider

        self.physical_size = storage_max
        self.physical_utilisation = 0
        self.virtual_allocation = 0
        self.content_type = ''

    def get_record(self, transient = True):
        retval = {'uuid': self.uuid,
                  'name_label': self.name_label,
                  'name_description': self.name_description,
                  'virtual_allocation': self.virtual_allocation,
                  'physical_utilisation': self.physical_utilisation,
                  'physical_size': self.physical_size,
                  'type': self.type,
                  'content_type': self.content_type,
                  'VDIs': self.images.keys()}

    def is_valid_vdi(self, vdi_uuid):
        return (vdi_uuid in self.images)

    def get_vdi_by_uuid(self, image_uuid):
        return self.images.get(image_uuid)

    def get_vdi_by_name_label(self, label):
        for image_uuid, image in self.images.items():
            if image.name_label == label:
                return image_uuid
        return None

    def get_vdis(self):
        return self.images.keys()

    def create_vdi(self, vdi_struct):
        raise NotImplementedError()

    def destroy_vdi(self, vdi_struct):
        raise NotImplementedError()
