#!/usr/bin/env python

import os
from os.path import *
from commands import getstatusoutput

#OVSConstants
OVS_BASE = "/OVS"
SR_REPO_META = ".ovsrepo"

#----------------------------------------------------------------------
def parse_df(_dir=""):
    cmd = "df --portability --print-type --local %s" % _dir
    l = []
    status, output = getstatusoutput(cmd)
    for line in output.splitlines()[1:]:
        line = line.strip()
        if not line: continue
        l.append(line.split())
    return l

#----------------------------------------------------------------------
from OVSFileSP import FileSP
class OVSSPLocalDir(FileSP):
    @classmethod
    def get_plugin_info(klass):
        return {
            "vendor name": "Oracle",
            "description": "Generic Local Filesystem Based Storage Provider",
            "plugin version": "0.0.1",
        }

    def __init__(self, fs_spec):
        #local directory. CANNOT be $OVS_BASE or its subdir.
        if not isdir(fs_spec):
            raise Exception("'%s' must be a directory." % fs_spec)
        _dir = "%s/" % normpath(fs_spec)
        if _dir.startswith("%s/" % OVS_BASE):
            raise Exception("'%s' cannot be '%s' or its subdirectory." % (
                    fs_spec, OVS_BASE))
        l = parse_df(fs_spec)
        if not l:
            raise Exception("'%s' is not on a local filesystem." % (
                    fs_spec,))

        #init
        self.fs_spec = fs_spec
        self.mount_links = []

    @property
    def info(self):
        return self.__dict__

    @property
    def fs_type(self):
        """
        """
        if hasattr(self, "_fs_type"):
           return self._fs_type

        l = parse_df(self.fs_spec)
        if not l:
            raise Exception("'%s' is not on a local filesystem." % (
                    self.fs_spec,))
        self._fs_type = l[0][1]

        return self._fs_type

    def mounted(self):
        """Mimic entry in /etc/mtab
        """
        l = []
        for _link in self.mount_links:
            l.append((self.fs_spec, _link, self.fs_type, "rw", "0", "0"))
        return l

    def mount(self, _dir):
        """Mount self.fs_spec to location _dir
        """
        _dir = normpath(_dir)
        if isdir(_dir):
            if os.listdir(_dir):
                return (1, "Mount point '%s' must be empty." % _dir)
            else:
                os.rmdir(_dir)

        cmd = "ln -sf %s %s" % (self.fs_spec, _dir)
        status, output = getstatusoutput(cmd)
        if not _dir in self.mount_links:
            self.mount_links.append(_dir)
        return (status, output)

    def umount(self, _dir=""):
        """Umount self.fs_spec from _dir
        """
        if not _dir:
            for _link in self.mount_links:
                _link = normpath(_link)
                os.remove(_link)
                self.mount_links.remove(_link)
        else:
            _link = normpath(_dir)
            if not exists(_link) or not islink(_link) or not os.readlink(_link) == self.fs_spec:
                return (1, "'%s' is not a mountpoint(link) of '%s'." % (
                        _dir, self.fs_spec))

            os.remove(_link)
            if _link in self.mount_links:
                self.mount_links.remove(_link)

        return (0, "")
