#!/usr/bin/env python

import random
import os
import commands
from os.path import realpath
from os.path import dirname, join, normpath
from glob import glob

from OVSPhySP import PhySP
class OVSSPiSCSI(PhySP):
    @classmethod
    def get_plugin_info(klass):
        info = {
        "vendor name": "Oracle",
        "description": "Generic iSCSI File Based Storage Provider",
        "plugin version": "0.0.1"
        }
        return info

    @classmethod
    def probe(klass, server, port=3260, username="", password=""):
        cmd = "iscsiadm --mode discovery --type sendtargets --portal %s:%s" % (server, port)
        (status, output) = commands.getstatusoutput(cmd)
        return (status, output)

    def __init__(self, server, port="3260", username=None, password=None):
        self.server = server
        self.port = port
        self.username = username
        self.password = password
        cmd = "iscsiadm --mode node --portal %s:%s --login" % (server, port)
        commands.getstatusoutput(cmd)

    @property
    def uuid(self):
        """Return UUID
        """
        if hasattr(self, "_uuid"):
            return self._uuid

        seed = "#OVSSPiSCSIOpenFiler#".join(map(str, [self.server, self.port, self.username, self.password]))
        random.seed(seed)
        _uuid = "%08x-%04x-%04x-%04x-%012x" % (
                random.randint(0, 0xffffffff),
                random.randint(0, 0xffff),
                random.randint(0, 0xffff),
                random.randint(0, 0xffff),
                random.randint(0, 0xffffffffffff))
        self._uuid = _uuid
        return self._uuid

    @property
    def info(self):
        return self.__dict__

    def list_devices(self):
        #list of [device, id, label, path, uuid]

        #by-path
        #ip-10.182.125.60:3260-iscsi-iqn.2006-01.com.openfiler:ovm3.ovm_g1
        pattern = "/dev/disk/by-path/ip-%s:%s-iscsi-iqn.*:*" % (self.server, self.port)
        l = glob(pattern)
        disk_by_path = dict([ (link, realpath(link)) for link in l])
        devs = disk_by_path.values()

        #by-id
        pattern = "/dev/disk/by-id/*"
        l = glob(pattern)
        disk_by_id = dict([(link, realpath(link)) for link in l if realpath(link) in devs])

        #by-label
        pattern = "/dev/disk/by-label/*"
        l = glob(pattern)
        disk_by_label = dict([(link, realpath(link)) for link in l if realpath(link) in devs])

        #by-uuid
        pattern = "/dev/disk/by-uuid/*"
        l = glob(pattern)
        disk_by_uuid = dict([(link, realpath(link)) for link in l if realpath(link) in devs])

        #filter result
        l = []
        for path, dev in disk_by_path.items():
            e = [dev, '', '', path, '']
            l.append(e)
            for id, _dev in disk_by_id.items():
                if _dev == dev:
                    e[1] = id
                    break
            for label, _dev in disk_by_label.items():
                if _dev == dev:
                    e[2] = label
                    break
            for uuid, _dev in disk_by_uuid.items():
                if _dev == dev:
                    e[4] = uuid
                    break
        return l

if __name__ == "__main__":
    print OVSSPiSCSI.get_plugin_info()
    print OVSSPiSCSI.probe("139.185.48.10")
    sp = OVSSPiSCSI(server="139.185.48.10")
    print sp.list_devices()

