#!/usr/bin/env python
#^#
# db_del.py
#
# <description>
#
#
# Copyright (C) 2006, 2007 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.
#$#

import sys
from os.path import *
sys.path.append(join(dirname(__file__), ".."))

import getopt
from OVSCDS import *

def db_del(db_name, key=""):
    d = db_dump(db_name)
    candidates = [k for k in d.keys() if key in k]

    if not candidates:
        print "No records match the key."
        return

    if len(candidates) == 1:
        k = candidates[0]
        db_remove(db_name, k)
        print "%s removed." % k
        return

    print "These records will be removed:"
    for n, k in enumerate(candidates):
        print "%d) %s" % (n, k)
    answer = raw_input("which one to remove? ('Y' to remove all) ")
    #remove all.
    if answer.lower() in ("y", "yes", "sure", "ok"):
        for n, k in enumerate(candidates):
            db_remove(db_name, k)
            print "Removed %d) %s" % (n, k)
        return
    #remove the chosen one.
    ns = answer.lower().replace(","," ").split()
    if not ns:
        print "Remove canceled."
        return
    for n in ns:
        try:
            n = int(n)
            k = candidates[n]
            db_remove(db_name, k)
            print "%s removed." % k
        except (ValueError, IndexError):
            print "Not a valid choice."

def usage():
    print "Usage: %s <[db_flags,]db_name> [<key>]" % sys.argv[0]

if __name__ == "__main__":
    try:
        opts, argv = getopt.gnu_getopt(sys.argv[1:], "h", ["help",])
    except getopt.GetoptError:
        usage()
        sys.exit(1)
    help = False
    for o, a in opts:
        if o in ("-h", "--help"):
            help = True

    if help or len(argv) < 1 or len(argv) > 2:
        usage()
        sys.exit(1)

    argv.extend([""] * (2 - len(argv)))
    db_name, key = argv
    if db_name.endswith(".db"):
        db_name = db_name[:-3]

    db_del(db_name, key)

