#!/usr/bin/env python
#^#
# db_kfv.py
#
# <description>
#
#
# Copyright (C) 2006, 2007 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.
#$#

import sys
from os.path import dirname, join
sys.path.append(join(dirname(__file__), ".."))

import getopt
from OVSCommons import *
from OVSCDS import *

def _str(v):
    if hasattr(v, "__dict__"):
        return repr(v.__dict__)
    else:
        return str(v)

def db_show(db_name, key, value_field=""):
    d = db_dump(db_name)
    for k in d:
        if key in k:
            if not value_field:
                v = d[k]
                if hasattr(v, "__dict__"):
                    print "%s\t=>\t%s" % (k, v.__dict__)
                else:
                    print "%s\t=>\t%s" % (k, v)
            else: #value must be a dict.
                v = d[k]
                if isinstance(v, dict):
                    print "%s\t=>\t%s" % (k, v[value_field])
                elif hasattr(v, "__dict__"):
                    print "%s\t=>\t%s" % (k, v.__dict__[value_field])
                else:
                    raise Exception("Error:\tvalue='%s' is not a dict. \n\tdb='%s', key='%s' " % (
                            _str(value), db_name, key))

def db_update(db_name, key, value, value_field="", value_type="str"):
    try:
        v_orig = db_load(db_name, key)
    except KeyError:
        print "Error: '%s' not found in db '%s'." % (key, db_name)
        return

    if value_type == "bool":
        if value == "False":
            value == False
        else:
            value == True
    else:
        value = eval("%s('%s')" % (value_type, value))

    if not value_field:
        print "old: %s => %s" % (key, _str(v_orig))
        db_save(db_name, key, value)
        print "new: %s => %s" % (key, _str(value))
    else:
        if isinstance(v_orig, dict):
            print "old: %s => %s" % (key, _str(v_orig[value_field]))
            v_orig[value_field] = value
        elif hasattr(v_orig, "__dict__"):
            print "old: %s => %s" % (key, _str(v_orig.__dict__[value_field]))
            v_orig.__dict__[value_field] = value
        else:
            raise Exception("Error:\tvalue='%s' is not a dict. \n\tdb='%s', key='%s' " % (
                    _str(value), db_name, key))

        db_save(db_name, key, v_orig)

        print "new: %s => %s" % (key, _str(value))

def db_kv(db_name, key, value, value_field="", value_type="str"):
    try:
        if not value:
            db_show(db_name, key, value_field)
        else:
            db_update(db_name, key, value, value_field, value_type)
    except Exception, e:
        if hasattr(e, "args") and isinstance(e.args, list):
            print e.args[0]
        else:
            print "%s: %s" % (e.__class__.__name__, e)

def usage():
    print "Usage: %s <[db_flags,]db_name> <key> [-f,--field field] [[-t,--type type] <value>]" % __file__

if __name__ == "__main__":
    try:
        opts, argv = getopt.gnu_getopt(sys.argv[1:], "hf:t:", ["help", "field=", "type="])
    except getopt.GetoptError:
        usage()
        sys.exit(1)
    help = False
    value_field = ""
    value_type = "str"
    for o, a in opts:
        if o in ("-h", "--help"):
            help = True
        elif o in ("-f", "--field"):
            value_field = a
        elif o in ("-t", "--type"):
            value_type = a

    if help or len(argv) < 2 or len(argv) > 3:
        usage()
        sys.exit(1)

    argv.extend([None] * (3 - len(argv)))
    db_name, key, value = argv
    if db_name.endswith(".db"):
        db_name = db_name[:-3]

    db_kv(db_name, key, value, value_field, value_type)

