#!/usr/bin/env python

#^#
# devinfo.py
#
# Copyright (C) 2009 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.
#$#

import os
import re
import stat
import commands

def getDevSize(path):
    cmd = "blockdev --getsize64 '%s'" % path
    (status, output) = commands.getstatusoutput(cmd)
    if status == 0:
        return int(output)
    return -1

def getDevReadonly(path):
    cmd = "blockdev --getro '%s'" % path
    (status, output) = commands.getstatusoutput(cmd)
    if status == 0 and re.match(r"^1$", output):
        return True
    return False

def getDevId(path):
    mpath = "dmsetup info --columns --noheadings -o uuid '%s'" % path
    scsi = "/lib/udev/scsi_id -g -u -s '/block/%s' -d '%s'" % (os.path.basename(path), path)
    ata = "/lib/udev/ata_id '%s'" % path
    cciss = "/sbin/cciss_id '%s'" % path
    dasd = "/sbin/dasd_id '%s'" % path
    edd = "/lib/udev/edd_id '%s'" % path
    for cmd in [mpath, scsi, ata, cciss, dasd, edd]:
        (status, output) = commands.getstatusoutput(cmd)
        if status == 0 and output:
            return output
    return ""

def getDevInfoXML(path):
    try:
        statinfo = os.stat(path)
        return "<blockdev>" + \
        "<id>%s</id>" % getDevId(path) + \
        "<path>%s</path>" % path + \
        "<readonly>%s</readonly>" % getDevReadonly(path) + \
        "<size>%s</size>" % getDevSize(path) + \
        "<major>%s</major>" % os.major(statinfo.st_rdev) + \
        "<minor>%s</minor>" % os.minor(statinfo.st_rdev) + \
        "</blockdev>"
    except Exception:
        return ""
