# Copyright (C) 2009 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.
'''
Created on 2009-7-7

@author: tizhang <tiger.zhang@oracle.com>
'''

from OVSWrappers import *
from OVSCDS import CDS
from OVSException import *
from OVSServerSelection import get_utility_server
from ServerModeHelper import _set_server_mode, _get_server_mode

def utl_set_server_mode(mode):
    try:
        OI(mode)
        _set_server_mode(mode)
        return ovs_success()
    except Exception, e:
        errmsg = fmt_err_msg(e)
        D(errmsg)
        return ovs_failed(errmsg)

def set_server_mode(server, mode):
    """
    Set server mode.

    @param server: server hostname
    @type server: string

    @param mode: 'maintain', 'normal'
    @type mode: string

    @return: success:<current mode>
            failed:<error message>
    """
    try:
        OI('%s,%s' % (server, mode))
        _, sp = get_utility_server(preference=server)
        return sp.utl_set_server_mode(mode)
    except Exception, e:
        errmsg = fmt_err_msg(e)
        D(errmsg)
        return ovs_failed(errmsg)

def utl_get_server_mode():
    try:
        OI('')
        mode = _get_server_mode()
        return ovs_success(mode)
    except Exception, e:
        errmsg = fmt_err_msg(e)
        D(errmsg)
        # fail back to 'normal'
        return ovs_success('normal')

def get_server_mode(server):
    """
    Get server mode.

    @param server: server hostname
    @type server: string

    @return: success:<current mode>
            failed:<error message>
    """
    try:
        OI(server)
        _, sp = get_utility_server(preference=server)
        return sp.utl_get_server_mode()
    except Exception, e:
        errmsg = fmt_err_msg(e)
        D(errmsg)
        return ovs_failed(errmsg)

if __name__ == '__main__':
    pass
