# Copyright (C) 2009 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.

from OVSWrappers import OI, ovs_success, ovs_failed, D
from OVSCDS import CDS, db_load
from OVSException import fmt_err_msg

def check_cluster_root_sr_new():
    """
    Check if there a cluster_root_sr_uuid_new field in cluster db
    
    @return: failed - no cluster_root_sr_uuid_new in cluster table.
             success - it's there.
    """

    try:
        sr_new = db_load('cluster', 'cluster_root_sr_uuid_new', get=True)
        if not sr_new:
            raise Exception("Didn't find cluster_root_sr_uuid_new in cluster table")
        return ovs_success()
    except Exception, ex:
        errmsg = fmt_err_msg(ex)
        D(errmsg)
        return ovs_failed(errmsg)
    
check_cluster_prerequisite = check_cluster_root_sr_new
        
def _get_cluster_root_sr():
    return db_load('cluster', 'cluster_root_sr_uuid')

def _get_sr(key=None):
    if key:
        return [db_load('sr', key),]
    else:
        cds = CDS('sr')
        return cds.db_sr.values()

def get_storage_repositories(cluster_root_only=False):
    """
    Get storage repositories information.
    
    @param cluster_root_only: if True, just get the root storage.
    @type cluster_root_only: boolean 
    @return success:<mount_dir>=<fs_spec>[;<mount_dir>=<fs_spec>]
    """
    try:
        OI(cluster_root_only)
        if cluster_root_only:
            sr = _get_sr(key=_get_cluster_root_sr())
        else:
            sr = _get_sr()
        ret = [ '%s=%s' % (c.mountpoint, c.sp.fs_spec) for c in sr ]
        return ovs_success(';'.join(ret))
    except Exception, ex:
        errmsg = fmt_err_msg(ex)
        D(errmsg)
        return ovs_failed(errmsg)