#!/usr/bin/env python
#^#
# Copyright (C) 2009 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.
#$#

from os.path import dirname, realpath
import sys
sys.path.append(dirname(dirname(realpath(__file__))))

import xmlrpclib
from getopt import getopt, GetoptError
from OVSProtocol import *
from do_rpc import get_local_agt_url
from OVSCDS import *
from OVSCommons import *
import tempfile

agt_url = get_local_agt_url()
sp = xmlrpclib.ServerProxy(agt_url)

#-----------------------------------------------------------------------

def config_o2cb(fs_spec):
    cmd = "debugfs.ocfs2 %s <<< stats 2>/dev/null | grep 'Feature Incompat'" % fs_spec
    try:
        output = old_run_cmd(cmd)
        if not "local" in output.lower():
            sp.cluster_prepare_ocfs2_conf(True)
            sp.clusterm_start_o2cb_service()
    except:
        pass

def mounted(fs_spec):
    f = open("/etc/mtab")
    m = []
    try:
        for line in f:
            l = line.split()
            if l[0] == fs_spec:
                m.append((l[0], l[1]))
    finally:
        f.close()
    return m

def get_repo_type(fs_spec):
    if not ":" in fs_spec:
        fs_spec = realpath(fs_spec)
        config_o2cb(fs_spec)

    global umount_flag
    global tmp_repo_dir
    global mount_point

    m = mounted(fs_spec)
    if not m:
        # mount -> get output -> umount
        tmp_repo_dir = tempfile.mkdtemp(prefix="ovsrepo_")
        mount_point = tmp_repo_dir
        run_cmd(["mount", fs_spec, tmp_repo_dir])
        umount_flag = True
    else:
        mount_point= m[0][1]

    output = run_cmd(["df", "-T", mount_point])
    l = output.splitlines()[-1].split()
    if l[0] == "ocfs2" or l[1] == "ocfs2":
        return "OVSSPOCFS2"
    elif l[0] == "nfs" or l[1] == "nfs":
        return "OVSSPNFS"
    else:
        return "OVSSPPartition"

#-----------------------------------------------------------------------

def new_repo(fs_type, fs_spec):
    rs = sp.sp_create(fs_type, fs_spec)
    sp_uuid = GETVAL(rs)["uuid"]
    rs = sp.sr_create(sp_uuid)
    sr_uuid = GETVAL(rs)["uuid"]
    print "[ NEW ] %s => %s" % (sr_uuid, fs_spec)
    return sr_uuid.encode("utf-8")

def set_root(sr_uuid):
    rs = sp.sr_set_root(sr_uuid)
    fs_spec = GETVAL(rs)["fs_spec"]
    print "[ R ] %s => %s" % (sr_uuid, fs_spec)

def list_repo():
    root_sr_uuid = db_load("cluster", "cluster_root_sr_uuid_new", get=True)
    root_sr_in_use = db_load("LOCAL_DB,node", "cluster_root_sr", get=True)
    if root_sr_in_use:
        root_sr_uuid_in_use = format_uuid(root_sr_in_use._uuid)
    else:
        root_sr_uuid_in_use = None

    d = db_dump("sr")
    for k, v in d.items():
        if k == root_sr_uuid_in_use:
            mark = "[ * ]"
        elif k == root_sr_uuid:
            mark = "[ R ]"
        else:
            mark = "[   ]"
        try:
            print "%s %s => %s" % (mark, k, v.sp.get_fs_spec())
        except:
            print "%s %s => %s" % (mark, k, "ERROR")

def del_repo(sr_uuid):
    sr = db_load("sr", sr_uuid)
    root_sr = db_load("LOCAL_DB,node", "cluster_root_sr", get=True)

    d = db_dump("srv")
    if d:
        if root_sr and root_sr._uuid == sr._uuid:
            yn = raw_input("""Deleting the root sr in use will cause the cluster configuration lost.
Please reassign the root sr before deleting if necessary.
Would you like to delete the root sr now? [y/N] """)
            if not yn.lower() in [ 'y', 'yes' ]:
                print "*** Abort deleting the root sr."
                return

    # delelte the root sr
    if root_sr and root_sr._uuid == sr._uuid:
        rs = sp.cluster_teardown(False)
        GETVAL(rs)
        print "*** Cluster teared down."
    # umount if necessary
    else:
        sp.sr_delete(sr_uuid)
        print "[ DEL ] %s => %s" % (sr_uuid, sr.sp.fs_spec)

def init_repo():
    d = db_dump("srv")
    uuid = db_load("cluster", "cluster_root_sr_uuid_new", get=True)

    if d and uuid:
        ha_enable = db_load("cluster", "pool_ha_enable", get=True)
        ha_enable = ha_enable or False
        rs = sp.cluster_setup(ha_enable)
    else:
        rs = sp.clusterm_init_sr()

    GETVAL(rs)
    print "*** Storage repositories initialized."

def usage():
    print """
Usage:
    %(prog)s -n <storage>               : create an sr
    %(prog)s -d <uuid>                  : delete an sr
    %(prog)s -r <uuid>                  : set root sr
    %(prog)s -l                         : list all sr
    %(prog)s -i                         : init all sr
    %(prog)s -h                         : print this help
Examples:
    %(prog)s --new ovs-test.cn.oracle.com:/nfs_share
    %(prog)s --new /dev/sde1
    %(prog)s --delete 2009664c-e447-4424-bfa4-9959c743faa9
    %(prog)s --root 2009664c-e447-4424-bfa4-9959c743faa9
    %(prog)s --list
    %(prog)s --init
    %(prog)s --help
Note:
    Deleting the root sr in use will cause the cluster configuration lost.
    Please reassign the root sr before deleting if necessary.

    [ * ] indicates the root sr in use,
    [ R ] indicates the root sr newly assigned.
""" % {'prog':sys.argv[0]}


def not_master():
    my_hostname = db_load("LOCAL_DB,node", "registered_hostname", get=True)
    master_hostname = db_load("cluster", "master_hostname", get=True)

    if my_hostname and master_hostname and my_hostname != master_hostname:
        return True
    else:
        return False


if __name__ == "__main__":
    try:
        opts, args = getopt(sys.argv[1:], "n:d:r:lih", ["new=", "delete=", "root=", "list", "init", "help"])
    except GetoptError:
        usage()
        sys.exit(2)

    if not opts or args:
        usage()
        sys.exit(2)

    try:
        sp.echo("whatever")
    except Exception, e:
        print "Cannot establish a connection to ovs-agent:", e
        sys.exit(3)

    action = None
    fs_type = None
    storage = None
    root_sr = None
    del_sr = None

    for o, a in opts:
        if o in ("-h", "--help"):
            usage()
            sys.exit(0)
        elif o in ("-l", "--list"):
            list_repo()
            sys.exit(0)
        elif o in ("-n", "--new"):
            action = "new"
            storage = a
            break
        elif o in ("-r", "--root"):
            action = "root"
            assert a in db_dump("sr"), "SR %s not found." % a
            root_sr = a
            break
        elif o in ("-d", "--delete"):
            action = "delete"
            assert a in db_dump("sr"), "SR %s not found." % a
            del_sr = a
            break
        elif o in ("-i", "--init"):
            action = "init"
            break
        else:
            assert False, "Option %s not recognized." % o

    # limit SR changes to the master server only
    if not_master():
        master_hostname = db_load("cluster", "master_hostname")
        print "SR changes are only allowed on the master server %s." % master_hostname
        sys.exit(4)

    global umount_flag
    global tmp_repo_dir
    global mount_point

    umount_flag = False
    tmp_repo_dir = ""
    mount_point = ""

    try:
        if action == "new":
            try:
                fs_type = get_repo_type(storage)
                new_repo(fs_type, storage)
            finally:
                if umount_flag:
                    run_cmd(["umount", "-fl", storage])
                if tmp_repo_dir:
                    run_cmd(["rm", "-fr", tmp_repo_dir])
        elif action == "root":
            set_root(root_sr)
        elif action == "delete":
            del_repo(del_sr)
        elif action == "init":
            init_repo()
        else:
            assert False, "Undefined operation."
    except KeyboardInterrupt:
        print "Operation cancelled."
        sys.exit(1)
    except Exception, e:
        print "Mount point: %s" % mount_point
        print "Error: %s" % e
        sys.exit(1)

    sys.exit(0)
