#!/usr/bin/env python
#^#
# rsh.py
#
# <description>
#
#
# Copyright (C) 2006, 2007 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.
#$#

import re
import sys
import socket
import pexpect
import os
from os.path import basename, exists, expanduser

def remove_known_host(hostname):
    if os.getuid() == 0:
        known_hosts = "/root/.ssh/known_hosts"
    else:
        known_hosts = expanduser("~/.ssh/known_hosts")
    if exists(known_hosts):
        f = open(known_hosts)
        try:
            lines = f.read().splitlines(True)
        finally:
            f.close()

        remote_hostname = ""
        if not re.match(r"^[0-9.]+$", hostname):
            remote_hostname = hostname.split(".")[0]
        remote_ip = socket.gethostbyname(hostname)

        _lines = []
        for line in lines:
            _continue = False
            for h in line.split()[0].split(","):
                if (remote_hostname and h.startswith(remote_hostname)) \
                        or socket.gethostbyname(h) == remote_ip:
                    _continue = True
            if _continue:
                continue
            _lines.append(line)

        f = open(known_hosts, "w")
        try:
            f.writelines(_lines)
        finally:
            f.close()

rsh_cmd = "ssh"
poss = [
    "(?i)password", #0, normal
    "(?i)Are you sure you want to continue connecting", #1,
    "(?i)permission denied", #2, wrong passwd?
    "(?i)connection closed by remote host", #3, close unexpectly
    "(?i)Name or service not known", #4, hostname cannot be resolved
    "(?i)No route to host", #5, the remote IP is unreachable
    "(?i)Connection Refused", #6,
    "(?i)Host key verification failed", #7
    pexpect.TIMEOUT, #8,
    pexpect.EOF, #9
]

def err_(msg):
    if isinstance(msg, str):
        msg = msg.replace("(?i)", "")
    elif msg == pexpect.TIMEOUT:
        msg = "timeout!"

    print >> sys.stderr, "Error: %s!" % msg

def nopasswd_sh(child):
    i = child.expect(poss, timeout=None)
    if i == 9:
        sys.exit(0)
    else:
        err_(poss[i])
        sys.exit(1)

def passwd_sh(child):
    child.sendline(passwd)
    nopasswd_sh(child)

if __name__ == "__main__":
    try:
        # parse parameter
        srv, cmd, passwd = sys.argv[1:]

        # remove the given server from known_hosts
        remote_host = srv
        if "@" in srv:
            remote_host = srv.split("@")[-1]
        if remote_host:
            try:
                remove_known_host(remote_host)
            except:
                pass
    except:
        print >> sys.stderr, "usage: %s <server> <cmd> <passwd>" % basename(__file__)
        sys.exit(1)

    # start working ...
    child = pexpect.spawn("%s %s %s" % (rsh_cmd, srv, cmd))
    i = child.expect(poss, timeout=30)

    if i == 0:
        passwd_sh(child)
    elif i == 1:
        child.sendline("yes")
        i = child.expect(poss, timeout=None)
        if i == 0:
            passwd_sh(child)
        elif i == 9:
            sys.exit(0)
        else:
            err_(poss[i])
            sys.exit(1)
    elif i == 9:
        sys.exit(0)
    else:
        err_(poss[i])
        sys.exit(1)
