#!/bin/bash
# Copyright (C) 2009 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.

# $1 - disk list file
# $2 - target image path

LC_ALL=C

ORIGINAL_IFS=$IFS
IFS=$'\n'

failed()
{
    echo "Failed"
    IFS=$ORIGINAL_IFS
    exit 1
}

usage()
{
    echo "Usage:"
    echo "$0 /path/to/vmdk/list/file /target/path"
    IFS=$ORIGINAL_IFS
    exit 1
}

if [[ $1 == "" || $2 == "" ]]; then
    usage
fi

if [[ $3 == "delete" ]]; then
    delete_src_images=true
fi

raw_tmp_dir="/OVS/tmp/v2v_raw_tmp.$$"

mkdir -p "$raw_tmp_dir"
mkdir -p "$2"

for disk in `cat $1`;
do
    echo "$disk"
    dir=`dirname "$disk"`
    filename=`basename "$disk"`
    diskname=`echo ${filename%.*}`
    tgt_img_path="$2/$diskname.img"

    rm "$tgt_img_path"

    echo "$filename"
    magic=`head -n 1 $disk`
    if [[ $magic == "# Disk DescriptorFile" ]]; then
        # splitted vmdk
        subdisks=`grep "\.vmdk\"" "$disk" | sed 's/^.*\"\(.*\)\".*/\1/'`

        first=1
        for i in $subdisks;
        do
            echo "Converting $i"
            cd "$raw_tmp_dir" && qemu-img convert -f vmdk "$dir/$i" "$i.raw"
            if [[ $? -ne 0 ]]; then
                rm "$tgt_img_path"
                cd "$raw_tmp_dir" && rm *.raw
                failed
            fi

            echo "Joining $i"
            if [[ $first == 1 ]]; then
                first=0
                cd "$raw_tmp_dir" && mv "$i.raw" "$tgt_img_path"
            else
                cd "$raw_tmp_dir" && cat "$i.raw" >> "$tgt_img_path" && rm "$i.raw"
            fi
            if [[ $? -ne 0 ]]; then
                rm "$tgt_img_path"
                cd "$raw_tmp_dir" && rm *.raw
                failed
            fi
            
            if [[ $delete_src_images ]]; then
                echo "Deleting $i"
                rm -f "$dir/$i"
            fi
        done
    else
        # flattend vmdk
        echo "Converting $disk"
        qemu-img convert -f vmdk "$disk" "$tgt_img_path"
        if [[ $? -ne 0 ]]; then
            failed
        fi
    fi

    if [[ $delete_src_images ]]; then
        echo "Deleting $disk"
        rm "$disk"
    fi
done
echo "Done"
IFS=$ORIGINAL_IFS
exit 0
