#!/bin/bash
# Copyright (C) 2009 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.


# $1 - disk list file
# $2 - target image path

LC_ALL=C

ORIGINAL_IFS=$IFS
IFS=$'\n'

failed()
{
    echo "Failed"
    IFS=$ORIGINAL_IFS
    exit 1
}

usage()
{
    echo "Usage:"
    echo "$0 /path/to/disk/list/file /target/path"
    IFS=$ORIGINAL_IFS
    exit 1
}

if [[ $1 == "" || $2 == "" ]]; then
    usage
fi

if [[ $3 == "delete" ]]; then
    delete_src_images=true
fi

mkdir -p "$2"

for disk in `cat $1`;
do
    echo "$disk"
    dir=`dirname "$disk"`
    filename=`basename "$disk"`
    tgt_img_path="$2/$filename.img"

    if [[ ${disk:(-4)} == ".vhd" ]]; then
        rm "$tgt_img_path"

        echo "Converting $filename"
        qemu-img convert -f vpc "$disk" -O raw "$tgt_img_path"
        if [[ $? -ne 0 ]]; then
            rm "$tgt_img_path"
            failed
        fi
    else
        echo "Copying $filename"
        if [[ "$disk" == "$2/$filename" ]]; then
            echo "Same file, skip."
        else
            cp "$disk" "$2/$filename"
            if [[ $? -ne 0 ]]; then
                failed
            fi
        fi
    fi

    if [[ $delete_src_images ]]; then
        echo "Deleting $disk"
        rm "$disk"
    fi
done
echo "Done"
IFS=$ORIGINAL_IFS
exit 0
